! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!+
! Module: pnback_mod
!
! Authors:  Brendan Perry     - NASA/GSFC, brendan.h.perry@nasa.gov (f95 rewrite)
!           Kip Kuntz         - NASA/GSFC/JHU (original algorithm)
!           Steven L. Snowden - NASA/GSFC (original algorithm)
!
! Description: Extended Source Analysis Software EPN background calculator
!
!
! Comments:
!
! $Id: pnback_mod.f90,v 1.1 2009/02/06 14:10:35 aibarra Exp $
!-
!  Creates particle background spectra and images for the EPN  
!
!  pnback was originally part of the XMM-ESAS package and it produces
!  model particle background spectra and images (in detector coordinates) 
!  from the output files of the perl script pnspectra (old pn-spectra).
!
!  Input:
!    inspecfile   : Input FOV spec file from pnspectra, OLDNEW: pnS001-obj.pi pnS001-fovt.pi
!    inspecoot    : Input OOT spec file from pnspectra, OLDNEW: pnS001-obj-oot.pi pnS001-fovoot.pi
!    outspecfile  : Input FOV spec file from pnspectra, OLDNEW: pnS001-back.pi pnS001-bkg.pi
!    outspecoot   : Input OOT spec file from pnspectra, OLDNEW: pnS001-obj-os.pi pnS001-bkgoot.pi
!    rmffile      : Input Response file from rmfgen         OLDNEW are the same: pnS001.rmf
!    withplotfile : create qdp output lightcurve/fitting file?
!    withimage    : create diagnostic image (true) or just spectra (false)?
!    elow         : Lower energy limit for the energy band in eV, not used if withimage=F
!    ehigh        : Upper energy limit for the energy band in eV, not used if withimage=F
!    quads    : boolean array (list) can use T/F/0/1 (which quadrant to use)
!    ***chisqfl  : boolean: r*4 create spec with unc? get right for sas-21???!!!
!    ***clobber: deprecated, uses SAS_CLOBBER
!    ***diag   : deprecated, uses SAS_VERBOSITY and withqdp param=T
!
! Examples:
! 1. pnback specfile=pnS001-obj.pi rmffile=pnS001.rmf elow=350 ehigh=1100 \
!    quads="1 1 1 1" chisqfl=T
! 2. pnback specfile=pnS001-obj.pi withqdp=F withimage=F elow=400 ehigh=1250 \
!    quads="T T F F" chisqfl=F
!
module pnback_mod

  use dal
  use cal
  use param
  use gofutils_mod
  use errorHandling
  use sasdal
  use gofio_mod
  use esas_mod

  implicit none

  private

  ! task parameter names

  character(len=*), parameter :: &
       inspecpar    = "inspecfile", &   ! was derived from prefix, i.e. S001/U006
       inootpar     = "inspecoot", &    ! was derived from prefix, i.e. S001/U006
       outspecpar   = "outspecfile", &  ! was derived, will be if 'default'
       outootpar    = "outspecoot", &   ! was derived, will be if 'default'              
       rmfsetpar    = "rmffile", &      ! was: parsed from prefix
       withqdppar   = "withplotfiles",& ! write out QDP diagnostic file?
       withimgpar   = "withimage",&     ! write out FITS image
       inimgpar     = "inimgfile",&     ! input FITS IMAGE
       inimgopar    = "inimgoot",&      ! input OOT FITS IMAGE
       outimgpar    = "outimgfile",&    ! output FITS IMAGE
       elowpar      = "elow", &         ! low energy cutoff
       ehighpar     = "ehigh", &        ! high energy cutoff
       quadspar     = "quads"           ! quads to apply
       !chisqflpar   = "chisqfl"        ! Use chi squared statistics, write out rate/error?
  !
  ! BHP: diag removed, replaced with SAS_VERBOSITY environment variable
  !
  public pnback

contains

  !---------------------------------------------------------------------
  subroutine pnback
    !---------------------------------------------------------------------
    !
    ! SAS datatypes:
    !
    ! SAS DATASET variables end with "_S"
    !
    type(DataSetT) :: inSPECO_S, inSPECOO_S, outSPECO_S, outSPECOO_S, inRMF_S, &
         inIMG_S, inIMGO_S, outIMG_S, inFWC_S, inFWCO_S, &
         QPB_S, specO_S, specOO_S, specOq_S, specOOq_S, specCFq_S, specCFOq_S, &
         specOCq_S, specOCOq_S, specOFq_S, specOFOq_S
    !
    ! SAS ARRAY variables end with "_A"
    !
    type(ArrayT)   :: inSPECO_A, inSPECOO_A, outSPECO_A, outSPECOO_A, inRMF_A, &
         inIMG_A, inIMGO_A, outIMG_A, inFWC_A, inFWCO_A, specO_A, &
         specOO_A, specOq_A, specOOq_A, specCFq_A, specCFOq_A, specOCq_A, &
         specOCOq_A, specOFq_A, specOFOq_A, QPB_A, dummy_A, arr
    !
    ! SAS TABLE variables end with "_T"
    !
    type(TableT)   :: inSPECO_T, inSPECOO_T, outSPECO_T, outSPECOO_T, inIMG_T, &
         inIMGO_T, inFWC_T, inFWCO_T, inRMF_T, outIMG_T, QPB_T, specO_T, &
         specOO_T, specOq_T, specOOq_T, specCFq_T, specCFOq_T, specOCq_T, &
         specOCOq_T, specOFq_T, specOFOq_T
    !
    ! SAS COLUMN variables end with "_C"
    !
    type(ColumnT)  :: channelO_C, channelOO_C, staterrO_C, staterrOO_C, &
         countsOR32_C, countsOOR32_C, countsOI32_C, countsOOI32_C, &
         countsI32_C, outSPECO_C, outSPECOO_C, channelOI16_C, channelOOI16_C, &
         channelOR32_C, channelOOR32_C
    !
    ! SAS BLOCK variables end with "_B"
    ! not needed
    ! type(BlockT) :: specIn_B, specOut_B, specOutO_B
    !
    character(len=256) :: inSPECO_F, inSPECOO_F, outSPECO_F, outSPECOO_F, inRMF_F, &
         qdpratehard_F, qdpaugspec_F, qdpraterev_F, qdpratehrev_F, qdpbridge_F,&
         qdpbackspec_F, qdpquadspec_F, qdpspec_F, inIMG_F, inIMGO_F, inFWC_F, &
         inFWCO_F, outIMG_F, QPB_F, specO_F, specOO_F, specOq_F, &
         specOOq_F, specCFq_F, specCFOq_F, specOCq_F, specOCOq_F, specOFq_F, &
         specOFOq_F, tempdir
    character(len=25) :: smodeB, smodeD, smodeO, smodeOO, smodeQ, smodeootQ
    character(len=24) :: title, radecsysiod, radecsysimg
    character(len=10) :: obsidB, obsidD, obsidO, obsidOO
    character(len=6) :: prefix, filtB, filtD, filtO, filtOO
    character(len=5) :: celow, cehigh, expidB, expidD, expidO, expidOO
    character(len=2) :: instB, instD, instO, instOO
    character(len=1) :: cquad
    !
    integer :: RMFrows, chansO, chansOO, chansTemp, chansoTemp, chansB, frmtimeO, &
         frmtimeOO, frmtimeB, frmtimeD, frmtimeTemp, nBlocks
    integer :: out(2), outl(2), outh(2), typeF, revlO, revlB, revlOO, revlD, nRowsO
    integer :: plc_al(2), plc_alsi(2), plc_aua(2), plc_aub(2), plc_auc(2), &
         plc_hi(2), plc_lo(2), plc_all(2), plc_alk(2), plc_hih(2), plc_loh(2), &
         plc_hard(2)
    integer :: elowO, ehighO, elowOO, ehighOO, elowD, ehighD, ehigh, elow, &
         elowB, ehighB, type, typeA, i_ok, pattern, patternOO
    integer :: i, i1, i2, ichan, ii, it, ites, itest, j, jj, k, n, nnn, status
    integer, dimension(2) :: band = (/0, 0/)
    integer :: ipix = 1, nels = 2, smo_fact = 12, nord
    integer, parameter :: specchans = 4096, xGood = 780, yGood = 780
    integer, dimension(2), parameter :: outDim = (/ 780,780 /)
    integer :: xSize, ySize
    integer, dimension(:,:), allocatable :: imgout, obimdet, imgfov    
    !  mine and maxe are the channel ranges for data not affected 
    !  by instrumental lines check these???!!!
    !    integer:: mine(2,2) = reshape((/160,350,950,2000/), (/2,2/))
    !    integer:: maxe(2,2) = reshape((/230,600,1030,2650/), (/2,2/))
    integer, dimension(4)   :: minel = (/160, 350,  950, 2000/)
    integer, dimension(4)   :: maxel = (/230, 600, 1030, 2650/)
    integer, dimension(2,2) :: mine
    integer, dimension(2,2) :: maxe
    integer :: lun, lunratehard, lunaugspec, lunraterev, lunratehrev, lunbridge, &
         lunbackspec, lunquadspec, lunspec
    integer(kind=int16), dimension(:), pointer :: channelO_IP, channelOO_IP, &
         chansO_IP, chansOO_IP
    integer(kind=int16), dimension(:,:), pointer :: aDI16
    integer(kind=int32), dimension(:), pointer :: specO_JP, specOO_JP, specOU_JP, &
         specOOU_JP, specOFq_JP, specOFOq_JP, specOCq_JP, specOCOq_JP, specCFq_JP, &
         specCFOq_JP, specOq_JP, specOOq_JP, countsO_JP, countsOO_JP
    integer(kind=int32), dimension(:,:), pointer :: aDI32, inIMG_JP, inIMGO_JP, &
         inFWC_JP, inFWCO_JP, outIMG_JP
    !
    logical :: withqdp, withimg, chisqfl, warndie, exists, firsttime, &
         quads(4)
    !
    real(kind=single), parameter :: amsca = 1440000.0
    real(kind=single) :: ymax = 0, ymin = 1.e8, aug_time = 1.0e6 ! keep hardcoded???!!!
    real(kind=single) :: t, t1, t2, t3, tot, tot0, tot1, tot2, tot3, totn, imR, imoR
    real(kind=single) :: tempsp, tempunc, tp
    real(kind=single) :: lim_area = 0.0 ! why???!!!
    real(single), allocatable, dimension(:,:) :: matrix
    real(kind=single) :: ootsca, unc, uncoot, expoO, backO, areaO, expoOO, backOO, &
         areaOO, expoB, backB, areaB, expoTemp, backTemp, areaTemp, expooTemp, &
         backoTemp, areaoTemp, hardratio, hardratioU, rateaug, ratecorq, ratecorqH, &
         ratecorqL, ratecorqT, ratecorqU, ratecorqV, cntscorband, cntscorq, &
         cntscorqH, cntscorqL, cntscorqV, cntscorqU, expo, expoN
    real(kind=single) aug_expo, aug_rate, avgNon0, bandl, bandh, hard, high
    real(kind=single), allocatable, dimension (:) :: expoOq_tot,areaOq_tot,backOq_tot, &
         expoOFq_tot, areaOFq_tot, backOFq_tot, expoOCq_tot, areaOCq_tot, backOCq_tot, &
         expoCFq_tot, areaCFq_tot, backCFq_tot, expoFq_tot, areaFq_tot, backFq_tot
    real(kind=single), allocatable, dimension (:) :: specO, specOO, specOU, specOOU, &
         specOCq, specOCqU, specOCOq, specOCOqU, specCFq, specCFqU, specTemp
    real(kind=single), allocatable, dimension (:) :: temp_top, temp_bot, temp_cor, &
         fillin, modl, aug_spec, obj_cnts, stat
    real(kind=single), dimension(3) :: aug_scal = (/0.00005, 0.2, 300.0/) !(/0.01, 1.0, 0.0/)
    real(kind=single), allocatable, dimension(:) :: eminrmf, emaxrmf, energy, &
         specFinal, specFinalU, maskFinal
    real(kind=single), allocatable, dimension(:,:) :: image, imageFinal
    real(kind=single), allocatable, dimension(:,:) :: specOq_tot,specOqU_tot, &
         specOFq_tot, specOFqU_tot, specOCq_tot, specOCqU_tot, specCFq_tot, &
         specCFqU_tot, specrati_tot, specmask_tot, maskOC_tot
    real(kind=single), dimension(:), pointer :: dummy_RP, specOU_RP, &
         specOOU_RP, specOCqU_RP, specOCOqU_RP, specCFqU_RP, specCFOqU_RP, &
         specOq_RP, specOOq_RP, specOFqU_RP, specOFOqU_RP, specOqU_RP, &
         specOOqU_RP, staterrO_RP, staterrOO_RP, countsO_RP, countsOO_RP
    !
    real(kind=single), dimension(10) :: coef ! why 10, besides it's what polyfit wants???!!!
    real(kind=single), allocatable, dimension(:) :: tempe, temps, tempu, ootspec, ootuncr
    real(kind=double), dimension(:,:), pointer :: aDR64
    real(kind=single), dimension(:,:), pointer :: aDR32, outIMG_RP
    !
    mine = reshape(minel, (/ 2, 2 /))
    maxe = reshape(maxel, (/ 2, 2 /))
    write(errstr,*) "Parsing input parameters..."
    call message(AppMsg,Noisy)
    !
    ! SS: First read in the object spectrum and get certain keywords
    chisqfl = .TRUE. ! fix for SAS-21???!!!
    !chisqfl = booleanParameter(chisqflpar) ! int vs real + unc output?
    ! object spectrum
    inSPECO_F = stringParameter(inspecpar) ! OLDNEW pnS001-obj.pi pnS001-fovt.pi
    write(errstr,*) "Opening observed spectral file: ",&
         trim(adjustl(inSPECO_F)) ! pnS001-fovtoot.pi from pnspectra
    call message(AppMsg,Always)  ! change to Noisy???!!!
    status = 0
    warndie = .TRUE.
    call check_open_set(trim(adjustl(inSPECO_F)),inSPECO_S,1,status,warndie) ! 1 = readonly
    write(errstr,*) "Getting FOV spectrum keywords..."
    call message(AppMsg,Always) ! change to Noisy???!!!
    status = 0
    warndie = .TRUE. ! if not found, unreadable, die
    typeF = 2 ! 1 = EVLI, 2 = SPEC, 3 = IMG, many of these are dummies
    call read_header_info(typeF,inSPECO_S,inSPECO_T,inSPECO_A,instO,obsidO,&
         expidO,smodeO,frmtimeO,filtO,revlO,backO,areaO,expoO,elowO,&
         ehighO,chansO,warndie)
    ! BHP: leave it open to the end to use as output template and transfer keywords
    !
    ! set prefix for default filename creation and other usage
    !
    if (instO .ne. 'PN') then 
       call error("NotPN","Input spectral file is not PN!")
    else 
       prefix = "pn" // trim(adjustl(expidO)) ! pnS001
       QPB_F = 'epn_qpb.fits.gz'
    endif
    !
    outSPECO_F   = stringParameter(outspecpar) ! OLDNEW: pnS001-back.pi pnS001-bkg.pi
    if (trim(adjustl(outSPECO_F)) .eq. 'default') then 
       outSPECO_F = trim(adjustl(prefix)) // "-bkg.pi" ! for now ???!!!
    endif
    ! BHP: get outspec file name first. If it exists and SAS_CLOBBER is not
    ! set, die early. Same for all output files below 
    call check_clobber(outSPECO_F) ! quit if clobber NOT set
    ! Now the OOT
    if (chisqfl) then ! only create OOT bkg spectrum if chisqfl is set
       outSPECOO_F   = stringParameter(outootpar) ! OLDNEW: pnS001-obj-os.pi/pnS001-bkgoot.pi 
       if (trim(adjustl(outSPECOO_F)) .eq. 'default') then 
          outSPECOO_F = trim(adjustl(prefix)) // "-fovtootsub.pi" ! for now ???!!!
       endif
       call check_clobber(outSPECOO_F) ! quit if clobber NOT set
    end if
    !
    ! SS: PN_BACK POSITION 01
    ! Next, read in the object OOT spectrum  and get certain keywords
    ! OLDNEW pnS001-obj-oot.pi pnS001-fovtoot.pi
    inSPECOO_F = stringParameter(inootpar) 
    if (trim(adjustl(inSPECOO_F)) .eq. 'default') then 
       inSPECOO_F = trim(adjustl(prefix)) // "-fovtoot.pi" ! for now ???!!!
    endif
    write(errstr,*) "Opening observed OOT spectral file: ",&
         trim(adjustl(inSPECOO_F)) ! pnS001-fovtoot.pi from pnspectra
    call message(AppMsg,Always) ! change to Noisy???!!!
    status = 0
    warndie = .TRUE.
    call check_open_set(trim(adjustl(inSPECOO_F)),inSPECOO_S,1,status,warndie) ! 1 = readonly
    !
    ! Get critical header keywords from spectra
    !
    ! BHP: Only have to read file once and keep values in memory.
    !      The old version opened and read it with fitsio 3 times.
    !      Read pnS002-fovt.pi (contains SPECTRUM extension,
    !      channel/counts columns, 4096 elements (typically))
    !
    ! SS: Read the observation chip spectra to get the exposure
    !
    ! SS:  Get information about the object spectrum and create an 
    !      OOT subtracted spectrum
    !
    ! SS: First read in the object spectrum and get certain keywords
    !

    ! SS: Read in the OOT events
    !
    ! SS: PN_BACK POSITION 03
    write(errstr,*) "Getting OOT spectrum keywords..."
    call message(AppMsg,Always) ! change to Noisy???!!!
    status = 0
    warndie = .TRUE. ! if not found, unreadable, die
    typeF = 2 ! 1 = EVLI, 2 = SPEC, 3 = IMG, many of these are dummies
    call read_header_info(typeF,inSPECOO_S,inSPECOO_T,inSPECOO_A,instOO,&
         obsidOO,expidOO,smodeOO,frmtimeOO,filtOO,revlOO,backOO,&
         areaOO,expoOO,elowOO,ehighOO,chansOO,warndie)
    ! Returned spectrum, specO_JP is an Int32 pointer (4096)
    ! Returned uncertainty, specOU_RP is sqrt(abs(real(specfov_JP)))
    ! or STAT_ERR if present (this routine is used by other code where this is true).
    !
    write(errstr,*) "Getting actual FOV spectrum from read_pha: "
    call message(AppMsg,Always) ! change to Noisy???!!!
    ! OLD/NEW: spec/uncr specO_JP/specOU_RP
    call read_pha(inSPECO_T,chansO,1,chansO_IP,specO_JP,dummy_RP,specOU_RP)
    ! BHP: keep open in memory, will use keywords from it for the template
    !
    ! SS: Read in the OOT events
    write(errstr,*) "Getting actual FOV OOT spectrum from read_pha: "
    call message(AppMsg,Always) ! change to Noisy???!!!
    ! OLD/NEW spec/uncr specOO_JP/specOOU_RP
    call read_pha(inSPECOO_T,chansOO,1,chansOO_IP,specOO_JP,dummy_RP,specOOU_RP)
    ! sanity check:
    if (chansOO .ne. chansO) then ! check always must be true???!!!
       call error("chansNEchans","Number of channels in OOT spectrum must be 4096!")
    end if
    ! set arrays for later usage (keeps indices between pointers in arrays in line
    allocate(specO(chansO),specOO(chansO))   ! OLD: spec/spec_oot
    allocate(specOU(chansO),specOOU(chansO)) ! OLD: uncr/uncr_oot
    do i=1,chansO
       specO(i)   = real(specO_JP(i-1)) ! OLD/NEW: spec/specO (first spec b4overwritelater)
       specOU(i)  = specOU_RP(i-1)      ! OLD/NEW: uncr/specOU(first uncr b4overwritelater)
       specOO(i)  = real(specOO_JP(i-1))! OLD/NEW: ootspec/specOO
       specOOU(i) = specOOU_RP(i-1)     ! OLD/NEW: ootuncr/specOOU
    enddo
    ! destroy no longer needed pointers ???!!!
    ! Check for QPB file. If doesn't exist or can't be read, no need to keep going
    write(errstr,*)"Running saslocate to find correct QPB cal file"
    call message(AppMsg,Always) ! change to Noisy!!!??? 
    call execute_command_line('saslocate lib/data/esas_data > temp.txt')
    lun = GetLun()
    open (unit=lun,file="temp.txt",status='old',access='sequential',&
         form='formatted',action='read',iostat=i_ok)
    if (i_ok == 0) then
       read(lun,'(A)') tempdir
       close (lun)
       call unlink ("temp.txt")
    else
       call error("saslocateBad","saslocate for QPB file failed!")
    end if
    ! Full path to QPB_F
    QPB_F = trim(adjustl(tempdir)) // "/" // trim(adjustl(QPB_F))
    write(errstr,*)"QPB calibration file:"
    call message(AppMsg,Always) ! change to Noisy!!!??? 
    write(errstr,*)trim(adjustl(QPB_F))
    call message(AppMsg,Always) ! change to Noisy!!!???
    !
    ! elow/ehigh are ONLY set if making diagnostic images. 
    ! setting them to zero to do that has been replaced by
    ! the withimage parameter
    !
    elow      = intParameter(elowpar) 
    ehigh     = intParameter(ehighpar)
    write(celow,'(I5)') elow
    write(cehigh,'(I5)') ehigh
    bandl = elow  / 1000.0
    bandh = ehigh / 1000.0
    !
    !
    inRMF_F = stringParameter(rmfsetpar) ! pnS001.rmf
    if (trim(adjustl(inRMF_F)) .eq. 'default') then
       inRMF_F = trim(adjustl(prefix)) // ".rmf"
       inquire(FILE=trim(inRMF_F),EXIST=exists)
       if (.not.exists) then
          write(errstr,*)"Derived RMF filename, " // trim(adjustl(inRMF_F)) // &
               " does not exist!"
          call error("noRMF",trim(adjustl(errstr)))
       endif
    end if
    !
    do i = 1, 4
       quads(i) = booleanParameter(quadspar,i-1) ! old int, now boolean param
    end do
    !
    withqdp = booleanParameter(withqdppar)  ! write diagnosis qdp files?
    if (withqdp) then
       ! this checks the QDP files first, most will be opened in augment_pn
       qdpratehrev_F = trim(adjustl(prefix))   // '-aughardrev.qdp' ! OLD: aug-rev-hard
       call check_clobber(qdpratehrev_F)
       qdpraterev_F = trim(adjustl(prefix))    // '-augraterev.qdp' ! OLD: aug-rev-rate 
       call check_clobber(qdpraterev_F)
       qdpaugspec_F = trim(adjustl(prefix))   // "-augspec.qdp" ! OLD: aug-spec
       call check_clobber(qdpaugspec_F)
       qdpbackspec_F = "pn" // trim(adjustl(expidO)) // "-bkgspec.qdp" ! OLD: back-spec
       call check_clobber(qdpbackspec_F)
       qdpbridge_F = trim(adjustl(prefix)) // '-bridgefit.qdp' ! OLD: bridge-fit
       call check_clobber(qdpbridge_F)
       qdpratehard_F = trim(adjustl(prefix))   // "-ratehard.qdp" ! OLD: aug
       call check_clobber(qdpratehard_F) 
       qdpquadspec_F = "pn" // trim(adjustl(expidO)) // "-quadspec.qdp" ! OLD: quad-spec
       call check_clobber(qdpquadspec_F) ! quit if clobber NOT set
       qdpspec_F = "pn" // trim(adjustl(expidO)) // "-spec.qdp" ! OLD: -spec
       call check_clobber(qdpspec_F) ! quit if clobber NOT set
    end if
    ! BHP: this replaces the if (eh>0) then make images schema
    withimg = booleanParameter(withimgpar) ! write out image files?
    if (withimg) then ! from pnspectra???!!! will overwrite???
       inIMG_F = stringParameter(inimgpar) ! pnS001-fovimdet-350-1100.fits
       if (trim(adjustl(inIMG_F)) .eq. 'default') then ! pnS001-bkgimdet-350-1100.fits
          inIMG_F = trim(adjustl(prefix)) // '-fovimdet-' // trim(adjustl(celow)) // &
               "-" // trim(adjustl(cehigh)) // ".fits"
       endif
       inIMGO_F = stringParameter(inimgopar) ! pnS001-fovimootdet-350-1100.fits
       if (trim(adjustl(inIMGO_F)) .eq. 'default') then
          inIMGO_F = trim(adjustl(prefix)) // '-fovimootdet-' // trim(adjustl(celow)) // &
               "-" // trim(adjustl(cehigh)) // ".fits"
       endif
       outIMG_F = stringParameter(outimgpar) ! pnS001-bkgimdet-350-1100.fits
       if (trim(adjustl(outIMG_F)) .eq. 'default') then 
          outIMG_F = trim(adjustl(prefix)) // '-bkgimdet-' // trim(adjustl(celow)) // &
               "-" // trim(adjustl(cehigh)) // ".fits"
       endif
       call check_clobber(outIMG_F) ! quit if clobber NOT set
    end if
    !
    call printparams (inSPECO_F,inSPECOO_F,outSPECO_F,outSPECOO_F,inRMF_F,&
         withqdp,withimg,inIMG_F,inIMGO_F,outIMG_F,elow,ehigh,quads,revlO,&
         obsidO,chisqfl)
    !
!    write(errstr,*) "Integer or real+unc output spectrum (chisqfl)?",chisqfl
!    call message(AppMsg,Always)
    if (withqdp) then
       write(errstr,*) "Output aug rate vs hardnress QDP plot : ",trim(qdpratehard_F) 
       call message(AppMsg,Always) ! change to Noisy???!!!
       write(errstr,*) "Output aug spec QDP plot              : ",trim(qdpaugspec_F) 
       call message(AppMsg,Always) ! change to Noisy???!!!
       write(errstr,*) "Output rate by revl QDP plot          : ",trim(qdpraterev_F) 
       call message(AppMsg,Always) ! change to Noisy???!!!
       write(errstr,*) "Output hard rate by revl QDP plot     : ",trim(qdpratehrev_F) 
       call message(AppMsg,Always) ! change to Noisy???!!!
       write(errstr,*) "Output bridge fit QDP plot            : ",trim(qdpbridge_F) 
       call message(AppMsg,Always) ! change to Noisy???!!!
       write(errstr,*) "Output bkg spec QDP plot              : ",trim(qdpbackspec_F) 
       call message(AppMsg,Always) ! change to Noisy???!!!
       write(errstr,*) "Output by quad spec QDP plot          : ",trim(qdpquadspec_F) 
       call message(AppMsg,Always) ! change to Noisy???!!!
    end if
    !
    !  Object (FOV) spectrum info read above for certain keywords
    !
    write(errstr,*)'Keywords from the spectral files: '
    call message(AppMsg,Always)
    write(errstr,*)'inst/instOOT       = ',instO," ",instOO
    call message(AppMsg,Always)
    write(errstr,*)'obsid/obsidOOT     = ',obsidO," ",obsidOO
    call message(AppMsg,Always)
    write(errstr,*)'expid/expidOOT     = ',expidO," ",expidOO
    call message(AppMsg,Always)
    write(errstr,*)'submode/submodeOOT = ',smodeO," ",smodeOO
    call message(AppMsg,Always)
    write(errstr,*)'chans/chansOOT     = ',chansO," ",chansOO
    call message(AppMsg,Always)
    write(errstr,*)'expo/expoOOT       = ',expoO," ",expoOO
    call message(AppMsg,Always)
    write(errstr,*)'back/backOOT       = ',backO," ",backOO
    call message(AppMsg,Always)
    write(errstr,*)'area/areaOOT       = ',areaO," ",areaOO
    call message(AppMsg,Always)
    write(errstr,*)'revl/revlOOT       = ',revlO," ",revlOO
    call message(AppMsg,Always)
!    write(errstr,*)"nchan, expoObs, backObs, backObs/amsca, areaObs, revlObs"
!    call message(AppMsg,Always)
!    write(errstr,2992) chansO, expoO, backO, backO/amsca, areaO, revlO
!    call message(AppMsg,Always)
!    write(errstr,*)"nchan, expoOOT, backOOT, backOOT/amsca, areaOOT, revlOOT"
!    call message(AppMsg,Always)
!    write(errstr,2992) chansOO, expoOO, backOO, backOO/amsca, areaOO, revlOO
!    call message(AppMsg,Always)
!2992 format(i5,f10.1,f12.0,f8.2,f8.3,i5)
    !
    ! SS: Set the PN mode
    ! BHP: Actually, set the PN OOT scale factor (getootsca is in gofutils)
    !
    warndie = .TRUE.
    call getootsca(smodeO,ootsca,warndie) ! in goflib/gofutils
    write(errstr,*)"PN submode/OOT Scale: ",trim(adjustl(smodeO)),"/",ootsca
    call message(AppMsg,Always) ! change to Noisy???!!!
    !
    ! chansO = number of channels in fov spectrum (should be 4096)
    ! or must = specchans = 4096???!!!
    !  read the rmf file, e.g.: pnS002.rmf
    !  response matrix file has 2 extensions: MATRIX and EBOUNDS.
    !  old subsn *read* MATRIX, but never used it. Other code does,
    !  so read_rmf returns a dummy matrix here as it's not used.
    ! check this???!!!
    ! Open and read RMF
    ! SS:  Get the channel to energy conversion from the rmf
    !
    write(errstr,*) "Open and read spectrum RMF: ",trim(adjustl(inRMF_F))
    call message(AppMsg,Always) ! change to Noisy???!!!
    status = 0
    warndie = .TRUE. ! will die if not found or unreadable
    call check_open_set(inRMF_F,inRMF_S,1,status,warndie) ! open for readonly
    allocate(eminrmf(chansO),emaxrmf(chansO),energy(chansO))
    eminrmf = 0. ! OLD: emin
    emaxrmf = 0. ! OLD: emax
    energy  = 0. ! OLD: ener
    RMFrows = 0  ! matrix below is just a dummy
    call read_rmf(inRMF_S,chansO,RMFrows,eminrmf,emaxrmf,matrix) ! old emin/emax
    call release(inRMF_S) ! done with rmf file, just needed emin/emax
    write(errstr,*)"Doing RMF channel to energy conversion..."
    call message(AppMsg,Always) ! change to Noisy???!!!
    ! SS: Get the channel to energy conversion from the rmf
    do i=1,chansO 
       energy(i) = (eminrmf(i) + emaxrmf(i))/2. ! OLD: ener/eminin/emaxin
       write(errstr,*)i,' ',energy(i)
       call message(AppMsg,Noisy)
       !   write(22,*)i,eminrmf(i),emaxrmf(i),energy(i)
    enddo
    deallocate(eminrmf,emaxrmf) ! arrays no longer needed
    !
    ! SS: Create the background spectrum by scaling the spectrum both by 
    !     the ratio of the exposure and the ratio of the backscale value,
    !     as well as the OOT scale factor.
    ! SECTION 1: Full observation data (NOT by quadrant) == O/OO OU/OOU suffixes
    !
    allocate(ootspec(chansO),ootuncr(chansO))
    do i=1,chansO
       ! OLD/NEW: ootspec/specOO
       ! OLD/NEW: obj_expo/expoO obj_back/backO
       ! OLD/NEW: expo_oot/expooot back_oot/backoot
       ! just repurpose specOO/specOOU, need them at the end of this program
       specOO(i) = ootsca * specOO(i) * &   ! OLD: ootspec/spec_oot
            (expoO/expoOO) * (backO/backOO) ! OLD: obj_expo/expo_oot/obj_back/back_oot
       ootspec(i) = ootsca * specOO(i) * &   ! OLD: ootspec/spec_oot
            (expoO/expoOO) * (backO/backOO) ! OLD: obj_expo/expo_oot/obj_back/back_oot       
       ! OLD/NEW: ootuncr/specOOU spec_oot/specoot_U
       ! OLD/NEW: obj_expo/expo_oot obj_back/back_oot
       ootuncr(i) = ootsca * specOOU(i) * &  ! OLD: ootuncr/uncr_oot
            (expoO/expoOO) * (backO/backOO)  ! OLD: obj_expo/expo_oot/obj_back/back_oot
       !       write(33,*)i,specOO(i),specOOU(i) 
    enddo
    !
    ! SS: If using chi-sq statistics, subtract the oot spectrum
    !
    allocate(obj_cnts(chansO),stat(chansO))
    do i=1,chansO ! OLD: nchan    
       if (chisqfl) then
          obj_cnts(i)  = specO(i) - specOO(i) ! OLD: obj_cnts/spec/ootspec
          stat(i) = sqrt(specOU(i)**2 + specOOU(i)**2) ! OLD: stat/uncr/ootuncr
       else
          obj_cnts(i) = specO(i)
          stat(i) = specOU(i)
       endif
       !  write(41,*)i,obj_cnts(i),stat(i)
    end do

    !
    write(errstr,*)'Sum of OBS counts    = ',sum(obj_cnts)
    call message(AppMsg,Always)
    write(errstr,*)'Sum of OOT counts    = ',sum(ootspec)
    call message(AppMsg,Always) 
    write(errstr,*)'Exposure             = ',expoO,'seconds'
    call message(AppMsg,Always)

    !
    ! BHP: canflag stuff removed from old pn_back as it's never used.
    ! should it be brought back???!!! canflag was hardcoded to FALSE
    ! then a large block of code was never done because of that.
    !
    ! SS: Now that we have the energy binning information, set some
    !  band limits for both evaluation bands and for detector 
    !  background line avoidance.
    !
    do i=1,chansO-1 ! was nchan

       ! SS: Set channel ranges for some energy bands, total, soft, hard

       if((energy(i) .lt. 0.4) .and. (energy(i+1) .ge. 0.4)) &
            plc_all(1) = i + 1
       if((energy(i) .le. 12.0) .and. (energy(i+1) .gt. 12.0)) &
            plc_all(2) = i
       if((energy(i) .lt. 0.6) .and. (energy(i+1) .ge. 0.6)) &
            plc_lo(1) = i + 1
       if((energy(i) .lt. 1.3) .and. (energy(i+1) .ge. 1.3)) &
            plc_lo(2) = i
       if((energy(i) .lt. 1.65) .and. (energy(i+1) .ge. 1.65)) &
            plc_hi(1) = i + 1
       if((energy(i) .lt. 7.2) .and. (energy(i+1) .ge. 7.2)) &
            plc_hi(2) = i
       if((energy(i) .lt. 9.2) .and. (energy(i+1) .ge. 9.2)) &
            plc_hih(1) = i + 1
       if((energy(i) .lt. 13.5) .and. (energy(i+1) .ge. 13.5)) &
            plc_hih(2) = i

       ! SS: Set channel ranges for the Al Kalpha and high energy 
       !  line contamination

       if((energy(i) .lt. 1.3) .and. (energy(i+1) .ge. 1.3)) &
            plc_alk(1) = i + 1
       if((energy(i) .le. 1.65) .and. (energy(i+1) .gt. 1.65)) &
            plc_alk(2) = i
       if((energy(i) .lt. 7.2) .and. (energy(i+1) .ge. 7.2)) &
            plc_hard(1) = i + 1
       if((energy(i) .le. 9.2) .and. (energy(i+1) .gt. 9.2)) &
            plc_hard(2) = i

       ! SS: Set channel ranges for energy band

       if((bandl .eq. 0.0) .and. (bandh .eq. 0.0)) then

          ! SS: If user defined band limits are 0 0 then set default values

          if((energy(i) .lt. 0.4) .and. (energy(i+1) .ge. 0.4)) &
               band(1) = i + 1
          if((energy(i) .le. 7.2) .and. (energy(i+1) .gt. 7.2)) &
               band(2) = i
       else

          ! SS: Set channel ranges for the user selected energy band

          if((energy(i) .lt. bandl) .and. (energy(i+1) .ge. bandl)) &
               band(1) = i + 1
          if((energy(i) .le. bandh) .and. (energy(i+1) .gt. bandh)) &
               band(2) = i
       endif

       !  SS: Set channel ranges for the interpolation bridge, currently the 
       !  same as the settings for the line contamination

       if((energy(i) .lt. 1.3) .and. (energy(i+1) .ge. 1.3)) &
            outl(1) = i + 1
       if((energy(i) .le. 1.65) .and. (energy(i+1) .gt. 1.65)) &
            outl(2) = i
       if((energy(i) .lt. 7.2) .and. (energy(i+1) .ge. 7.2)) &
            outh(1) = i + 1
       if((energy(i) .le. 9.2) .and. (energy(i+1) .gt. 9.2)) &
            outh(2) = i
    enddo
    ! change these to Noisy???!!!
    write(errstr,*)'Total channels in obs spectra  : ', chansO
    call message(AppMsg,Always)
    write(errstr,*)'Total band channel range       : ', plc_all
    call message(AppMsg,Always)
    write(errstr,*)'Soft band channel range        : ', plc_lo
    call message(AppMsg,Always)
    write(errstr,*)'Hard band channel range        : ', plc_hi
    call message(AppMsg,Always)
    write(errstr,*)'Hardest band channel range     : ', plc_hih
    call message(AppMsg,Always)
    write(errstr,*)'Selected band channel range    : ', band
    call message(AppMsg,Always)
    write(errstr,*)'Low bridge band channel range  : ', outl
    call message(AppMsg,Always)
    write(errstr,*)'High bridge band channel range : ', outh
    call message(AppMsg,Always)
    !    
    ! SS: Read the observation chip spectra to get the exposure.  The
    !     code also calculates the OOT subtracted spectra and uncertainty.
    !     but the program doesn't use them for anything.
    ! SS: The obj data are the observation data
    ! BHP: So, should it be removed???!!!
    !
    write(errstr,*)' Process the individual quadrant observation data ', &
         ' and create OOT subtracted '
    call message(AppMsg,Always)
    write(errstr,*)' spectra and exposures which are area ', &
         'and exposure normalized.'
    call message(AppMsg,Always)

    !
    ! process the by-quadrant fov and oot data, create:
    ! specOQ(4096,4), only fill where quad=T
    ! specOQ_U(4096,4), the uncertainties on specfovQ
    ! q stands for quadrant, _U for uncertainties
    !
    allocate(specOq_tot(chansO,4),specOqU_tot(chansO,4))
    allocate(expoOq_tot(4),areaOq_tot(4),backOq_tot(4))
    specOq_tot  = 0. ! (4096,4) OLD: obs_spec check these???!!!
    specOqU_tot = 0. ! (4096,4) OLD: obs_uncr
    expoOq_tot  = 0. ! OLD: reg_exp_obs
    backOq_tot  = 0. ! OLD: reg_area_obs
    !
    ! SS: The obj data are the observation data (by-quad)
    ! SECTION 2: Full observation data by-quad Oq/OOq suffix
    ! TYPE: Oq/OOq suffix spec+Obj+quad/spec+Obj+OOT+quad
    do i=1,4 ! begin do loop by quadrant fov/oot data
       if(quads(i)) then ! BHP: boolean now
          revlD     = 0 
          backTemp  = 0.
          areaTemp  = 0.
          expoTemp  = 0.
          backoTemp = 0.
          areaoTemp = 0.
          expooTemp = 0.
          ! FOV by-quadrant spec OLDNEW: pnS001-3obj.pi/pnS001-fovq3.pi
          write(cquad,'(I1)')i
          specOq_F = "pn" // trim(adjustl(expidO)) // "-" // &
               "fovq" // trim(adjustl(cquad)) // ".pi"
          write(errstr,*)"Current Quad spectral file: ",trim(specOq_F)
          call message(AppMsg,Always) ! change to Noisy???!!!
          status   = 0
          warndie  = .TRUE. ! dies if not found/readable
          ! code to skip not found/unreadable and press on without as below???!!!
          ! SS: PN_BACK POSITION 04
          call check_open_set(specOq_F,specOq_S,1,status,warndie) 
          ! only need back/expo/area from these spec files (beside spec itself),
          ! so, dummy variables for the rest
          typeF = 2
          status   = 0
          warndie  = .TRUE.
          call read_header_info(typeF,specOq_S,specOq_T,dummy_A,instD,& ! D for dummy
               obsidD,expidD,smodeD,frmtimeD,filtD,revlD,backTemp,areaTemp,&
               expoTemp,elowD,ehighD,chansTemp,warndie) ! dummy for check, not used
          ! OLD/NEW: spec/specOq_JP/specOqU_RP
          call read_pha(specOq_T,chansTemp,1,chansO_IP,specOq_JP,dummy_RP,specOqU_RP)
          ! sanity checks, put in???!!!
          !
          ! SS: Now do the OOT events (PN_BACK POSITION 04)
          ! OLDNEW: pnS001-3obj-oot/pnS001-fovootq3.pi
          !
          specOOq_F = "pn" // trim(adjustl(expidO)) // "-" // & 
               "fovootq" //  trim(adjustl(cquad)) // ".pi"
          write(errstr,*)"Current Quad OOT spectral file: ",trim(specOOq_F)
          call message(AppMsg,Always)
          status  = 0
          warndie = .TRUE. 
          call check_open_set(specOOq_F,specOOq_S,1,status,warndie)
          ! only getting expo/area data from these spec files???!!!
          typeF = 2
          status   = 0
          warndie  = .TRUE.
          call read_header_info(typeF,specOOq_S,specOOq_T,dummy_A,instD,& ! D=dummy
               obsidD,expidD,smodeD,frmtimeD,filtD,revlD,backoTemp,areaoTemp,&
               expooTemp,elowD,ehighD,chansoTemp,warndie) ! mostly dummy, not used
          ! OLD/NEW: spec_oot/uncr_oot specootQ_JP/specootQU_RP
          call read_pha(specOOq_T,chansoTemp,1,chansOO_IP,specOOq_JP,dummy_RP,specOOqU_RP)
          ! sanity checks, put in???!!!
          !          
          ! fill exposure/area array for each selected quad
          !

          if ((backTemp .gt.0) .and. (expoTemp .gt. 0) .and. &
               (backoTemp .gt. 0) .and. (expooTemp .gt. 0) ) then 
             expoOq_tot(i) = expoTemp ! OLD: reg_exp_obs
             areaOq_tot(i) = backTemp ! OLD: reg_area_obs
             ! change to Noisy???!!!         
             !             write(errstr,*)"FOV values for Quad     : ",i
             !             call message(AppMsg,Always)
             write(errstr,*)"chansFOV,expoFOV,backFOV,backFOV/amsca,areaFOV,revolutFOV"
             call message(AppMsg,Always)          
             write(errstr,4004)chansO,expoTemp,backTemp,backTemp/amsca,areaTemp,revlD
             call message(AppMsg,Always)
             !             write(errstr,*)"FOV OOT values for Quad : ",i
             !             call message(AppMsg,Always)
             write(errstr,*)"chansOOT,expoOOT,backOOT,backOOT/amsca,areaOOT,revolutOOT"
             call message(AppMsg,Always)          
             write(errstr,4004)chansoTemp,expooTemp,backoTemp,backoTemp/amsca,areaoTemp,revlD
             call message(AppMsg,Always)
4004         format(i5,1x,f9.1,1x,f12.1,1x,f7.2,1x,f7.1,1x,i6)
             !
             ! create the FOV OOT subtracted spectrum huh???
             ! OLD/NEW: obs_spec = specOq  (newly allocated array above)
             ! OLD/NEW: obs_uncr = specOqU (newly allocated array above)
             ! OLD/NEW: spec     = specOq_JP (int32 pointer)
             ! OLD/NEW: spec_oot = specOOq_JP (int32 pointer)
             ! OLD/NEW: back     = backTemp, expo = expoTemp
             ! OLD/NEW: back_oot = backoTemp, expo_oot = expooTemp
             ! OLD/NEW: uncr     = specOqU_RP (real32 pointer)
             ! OLD/NEW: uncr_oot = specOOqU_RP (real32 pointer)
             ! SS: PN_BACK POSITION 05
             do ichan=1,chansO ! (4096,4)
                ! write(36,*)specOq_JP(ichan-1),specOOq_JP(ichan-1),&
                !specOqU_RP(ichan-1),specOOqU_RP(ichan-1)
                specOq_tot(ichan,i) =  & ! OLD: obs_spec
                     amsca * (real(specOq_JP(ichan-1))/backTemp/expoTemp - &
                     ootsca * real(specOOq_JP(ichan-1))/backoTemp/expooTemp)
                specOqU_tot(ichan,i) = & ! OLD: obs_uncr
                     amsca * (sqrt((specOqU_RP(ichan-1))**2 +  &
                     (ootsca * specOOqU_RP(ichan-1) * (backTemp/backoTemp)* &
                     (expoTemp/expooTemp))**2)/backTemp/expoTemp)
                ! write(34,*)i,ichan,specOq_tot(ichan,i),specOqU_tot(ichan,i)
             enddo
          else
             write(errstr,*)"Quad ",cquad,&
                  " has back or expo = 0, removing it from analysis"
             call message(AppMsg,Always)
             quads(i) = .FALSE.
          endif
       endif
       call release(specOq_S)
       call release(specOOq_S)
    enddo ! end do loop by-quad FOV data

    ! deallocate everything no longer needed here???!!!
    !
    write(errstr,*)'Backscale and exposure from the observation data'
    call message(AppMsg,Always)
    write(errstr,*)"FOV area, backscale, exposure:"
    call message(AppMsg,Always)
    do i=1,4
       if (quads(i)) then ! BHP: a boolean now
          write(errstr,2997) i, areaOq_tot(i), areaOq_tot(i)/amsca, expoOq_tot(i)
2997      format(' Quadrant',i2,'  backscale=',f10.0, ' backscale/amsca =',&
               ' ',f7.2,'  exposure =',f9.1)
          call message(AppMsg,Always) ! change to Noisy???!!!
       endif
    enddo
    !
    ! SECTION 3: Full observation FWC data by-quad OFq/OFOq suffix
    !
    ! SS: Read the fwc spectra from the object region
    ! Figure out which quadrants are involved
    ! Use reg_area as a vector describing which chips are in use
    ! SS: The ff data are the FWC data from the object region
    ! OLDNEW: pnS001-3ff.pi pnS001-fwcq3.pi.
    ! FWC data are in the CCF, these were created by pnspectra using that data. 
    !
    write(errstr,*)'Process the FWC data from the object region and ', &
         'create OOT '
    call message(AppMsg,Always) ! change to Noisy???
    write(errstr,*) 'subtracted and area and exposure normalized ', &
         'spectra'
    call message(AppMsg,Always) ! change to Noisy???
    write(errstr,*) ' '
    call message(AppMsg,Always) ! change to Noisy???
    !write(errstr,*) 'List individual file names and nchan, expo, ', &
    !     'back, back/amsca, area, revl, err'
    !call message(AppMsg,Verbose) ! change to Noisy???
    ! read in the by-quadrant fwc and fwc oot data, create:
    ! specOFq(4096,4), only fill the 4 where quad=T
    ! specOFqU(4096,4), the uncertainties on specOFq
    ! area and expo are r*4 x 4 elements, one for each quad
    !
    allocate(expoOFq_tot(4),areaOFq_tot(4))
    allocate(specOFq_tot(chansO,4),specOFqU_tot(chansO,4))
    expoOFq_tot  = 0. ! OLD: reg_exp_fwc check these???!!!
    areaOFq_tot  = 0. ! OLD: reg_area_fwc
    specOFq_tot  = 0. ! (4096,4) OLD: reg_spec
    specOFqU_tot = 0. ! (4096,4) OLD: reg_uncr
    ! TYPE: OFq/OFOq suffix Obs+FWC+quad
    do i=1,4 ! begin do loop by quadrant FWC/FWCOOT data
       ! SS: Read in the normal data
       if(quads(i)) then ! BHP: boolean now
          revlD     = 0 
          backTemp  = 0.
          areaTemp  = 0.
          expoTemp  = 0.
          backoTemp = 0.
          areaoTemp = 0.
          expooTemp = 0.
          write(cquad,'(I1)')i
          ! FOV by-quadrant spec OLDNEW pnS001-3ff.pi/pnS001-fovq3.pi
          specOFq_F = "pn" // trim(adjustl(expidO)) // "-" // &
               "fovfwcq" // trim(adjustl(cquad)) // ".pi"
          write(errstr,*)"Current quad spectral file: ",trim(specOFq_F)
          call message(AppMsg,Always)
          status   = 0
          warndie  = .TRUE. ! dies if not found/readable
          call check_open_set(specOFq_F,specOFq_S,1,status,warndie)
          ! only need back/expo/area from these spec files (beside spec itself),
          ! so, dummy variables for the rest
          typeF = 2
          status   = 0
          warndie  = .TRUE. ! die if can't find/read PN_BACK POSITION 06
          call read_header_info(typeF,specOFq_S,specOFq_T,dummy_A,instD,& ! D for dummy
               obsidD,expidD,smodeD,frmtimeD,filtD,revlD,backTemp,areaTemp,&
               expoTemp,elowD,ehighD,chansTemp,warndie) ! dummy for check, not used
          ! OLD/NEW: spec/uncr specOFq_JP/specOFqU_RP
          call read_pha(specOFq_T,chansO,1,chansO_IP,specOFq_JP,dummy_RP,specOFqU_RP)
          ! sanity checks, put some in here???!!!
          !
          ! SS:  Read in the OOT data
          ! OLDNEW: pnS001-3ff-oot/pnS001-fwcootq3.pi
          ! spec+FWC+OOT+quad
          specOFOq_F = "pn" // trim(adjustl(expidO)) // "-" // & 
               "fovfwcootq" //  trim(adjustl(cquad)) // ".pi"
          write(errstr,*)"Current quad OOT spectral file: ",trim(specOFOq_F)
          call message(AppMsg,Always)
          status  = 0
          warndie = .TRUE. 
          call check_open_set(specOFOq_F,specOFOq_S,1,status,warndie) 
          ! only getting expo/area data from these spec files???!!!
          typeF = 2
          status   = 0
          warndie  = .TRUE.
          call read_header_info(typeF,specOFOq_S,specOFOq_T,dummy_A,instD,& ! D for dummy
               obsidD,expidD,smodeD,frmtimeD,filtD,revlD,backoTemp,areaoTemp,&
               expooTemp,elowD,ehighD,chansoTemp,warndie) ! mostly dummy, not used
          ! OLD/NEW: spec_oot/uncr_oot specootQ_JP/specootQU_RP
          call read_pha(specOFOq_T,chansO,1,chansO_IP,specOFOq_JP,dummy_RP,specOFOqU_RP)
          ! sanity checks, put some in here???!!!
          !          
          ! fill exposure/area array for each selected quad
          !
          expoOFq_tot(i) = expoTemp ! OLD: reg_exp_fwc
          areaOFq_tot(i) = backTemp ! OLD: reg_area_fwc
          expoN = expoTemp
          !          
          write(errstr,*)"FWC values for Quad ",cquad
          call message(AppMsg,Always)
          write(errstr,*)"chansFWC,expoFWC,backFWC,backFWC/amsca,areaFWC"
          call message(AppMsg,Always)          
          write(errstr,4005)chansO,expoTemp,backTemp,backTemp/amsca,areaTemp
          call message(AppMsg,Always)
          write(errstr,*)"FWC OOT values for Quad ",i
          call message(AppMsg,Always)
          write(errstr,*)"chansOOT,expoOOT,backOOT,backOOT/amsca,areaOOT"
          call message(AppMsg,Always)          
          write(errstr,4005)chansoTemp,expooTemp,backoTemp,backoTemp/amsca,areaoTemp
          call message(AppMsg,Always)
4005      format(i5,1x,f9.1,1x,f12.1,1x,f7.2,1x,f7.1,1x)
          !
          ! create the FWC OOT subtracted spectra
          ! OLD/NEW: reg_spec = specOFq_tot  (newly allocated array above)
          ! OLD/NEW: reg_uncr = specOFqU_tot (newly allocated array above)
          ! OLD/NEW: spec     = specOFq_JP (int32 pointer)
          ! OLD/NEW: spec_oot = specOFOq_JP (int32 pointer)
          ! OLD/NEW: back     = backTemp, expo = expoTemp
          ! OLD/NEW: back_oot = backoTemp, expo_oot = expooTemp
          ! OLD/NEW: uncr     = specOFqU_RP (real32 pointer)
          ! OLD/NEW: uncr_oot = specOFOqU_RP (real32 pointer)
          ! OLD/NEW: reg_area_fwc = areaOFq_tot
          !
          if ((backTemp .gt.0) .and. (expoTemp .gt. 0)) then
             ! SS: There is active area on this chip, normalize the spectrum
             !  and subtract the OOT data.  Data are normalized to per area
             !  per exposure
             do ichan=1,chansO ! (4096,4)
                specOFq_tot(ichan,i) =  & ! OLD: reg_spec
                     amsca * (real(specOFq_JP(ichan-1))/backTemp/expoTemp - &
                     ootsca * (real(specOFOq_JP(ichan-1))/backoTemp/expooTemp))
                specOFqU_tot(ichan,i) = & ! OLD: reg_uncr
                     amsca * (sqrt((specOFqU_RP(ichan-1))**2 +  &
                     (ootsca * specOFOqU_RP(ichan-1) * (backTemp/backoTemp)* &
                     (expoTemp/expooTemp))**2)/backTemp/expoTemp)
                ! these start at channel 31 (old=61) and are much higher than the 'old'
                ! until about channel 80
                !  write(37,*)i,ichan,specOFq_tot(ichan,i),specOFqU_tot(ichan,i)
                !  write(39,*)i,ichan,specOFq_JP(ichan),specOFqU_RP(ichan)
             enddo
          else
             specOFq_tot    = 0.       ! OLD: reg_spec
             specOFqU_tot   = 0.       ! OLD: reg_uncr
             areaOFq_tot(i) = backTemp ! OLD: reg_area_fwc
             write(errstr,*)"Quad ",cquad,&
                  " has back or expo = 0, removing it from analysis"
             call message(AppMsg,Always)
             quads(i) = .FALSE.
          endif
       endif
       call release(specOFq_S)
       call release(specOFOq_S)
       ! OLD approx PN_BACK POSITION 07 in original code
    enddo ! end do loop FWC by-quad data
    !
    write(errstr,*) 'Region backscale and exposure from the FWC data:'
    call message(AppMsg,Always) ! change to verbose???!!!
    write(errstr,*) 'Quad, areafwcQuad, areafwcQuad/amsca, expofwcQuad'
    call message(AppMsg,Always) ! change to verbose???!!!
    do i=1,4
       if (quads(i)) then
          write(errstr,2997) i,areaOFq_tot(i),areaOFq_tot(i)/amsca,expoOFq_tot(i)
          call message(AppMsg,Always) ! change to verbose ???!!!
       end if
    end do

    !
    ! SS: Remove any quadrant that has less area than the limiting area, lim_area
    ! BHP: lim_area hardcoded to 0.0 at top???!!!
    !
    do i=1,4
       if(quads(i)) then
          if((areaOFq_tot(i) .lt. lim_area) .or. & ! OLD: reg_area_fwc(i)
               (areaOFq_tot(i) .eq. 0.0) .or.  &   ! OLD: reg_area_fwc(i)
               (areaOq_tot(i) .eq. 0.0)) then      ! OLD: reg_area_obs(i)
             areaOFq_tot(i) = 0.0 ! OLD: reg_area_fwc
             expoOFq_tot(i) = 0.0 ! OLD: reg_exp_fwc
             do j=1,chansO
                specOFq_tot(j,i)  = 0.0   ! OLD: reg_spec
                specOFqU_tot(j,i) = 0.0   ! OLD: reg_uncr
             end do
          endif
       endif
    enddo
    !
    !  SS: Read corner spectra from the observation
    !  First set up the spectrum masks
    !  This eliminates any deselected quadrants
    !
    !  TYPE: OC suffix Obs+Corner no quad
    allocate(maskOC_tot(chansO,4)) ! (4096,4)
    maskOC_tot = 0.
    do i=1,4
       ! no need to do loop chansO
       if(quads(i)) then
          maskOC_tot = 1. ! OLD: oc_mask (4096,4)
       endif
    enddo
    !
    ! SS: The oc data are observation data from the corners
    ! SECTION 4: Full observation Corner data by-quad OCq/OCOq suffix
    !
    ! Read in the by-quadrant corner and corner oot data, create:
    ! speccorq_tot(4096,4), only fill the 4 where quad=T
    ! speccorqU_tot(4096,4), the uncertainties on speccorQ
    ! area and expo are r*4 x 4 elements, one for each quad
    !
    write(errstr,*)'Process the observation data from the corner ', &
         'regions and create '
    call message(AppMsg,Always)
    write(errstr,*)'OOT subtracted and area and exposure ', &
         'normalized spectra '
    call message(AppMsg,Always)
    !
    ! Finally going to need QPB file opened and passed to augment_pn
    status = 0
    warndie = .TRUE.
    call check_open_set(trim(QPB_F),QPB_S,1,status,warndie)
    ! allocate arrays for by-quad work
    allocate(specOCq_tot(chansO,4),specOCqU_tot(chansO,4)) ! all quad's spec
    allocate(specOCq(chansO),specOCqU(chansO))        ! temp each quad's spec
    allocate(specOCOq(chansO),specOCOqU(chansO))      ! temp each quad's OOT spec
    allocate(specTemp(chansO),aug_spec(chansO))
    allocate(expoOCq_tot(4),areaOCq_tot(4))
    expoOCq_tot  = 0.  ! OLD: cor_exp_obs
    areaOCq_tot  = 0.  ! OLD: cor_area_obs
    specOCq      = 0.  ! OLD: spec(4096)
    specOCqU     = 0.  ! OLD: uncr(4096)
    specOCq_tot  = 0.  ! OLD: oc_spec(4096,4)
    specOCqU_tot = 0.  ! OLD: oc_uncr(4096,4)
    firsttime = .TRUE.
    ! TYPE: OCq/OCOq Obs+Corner+by-quad
    do i=1,4 ! loop through quads with the corner spectra
       if( quads(i) ) then ! BHP: boolean now
          write(cquad,'(I1)')i
          ! by-quadrant spec OLDNEW: pnS002-3oc.pi/pnS002-corq3.pi
          specOCq_F = "pn" // trim(adjustl(expidO)) // "-corq" // &
               trim(adjustl(cquad)) // ".pi"
          ! SS: Read in the normal data
          write(errstr,*)"Current CORNER quad spectral file: ",trim(specOCq_F)
          call message(AppMsg,Always)
          backTemp  = 0.
          areaTemp  = 0.
          expoTemp  = 0.
          backoTemp = 0.
          areaoTemp = 0.
          expooTemp = 0.
          status  = 0
          warndie = .TRUE. ! PN_BACK POSITION 08
          call check_open_set(specOCq_F,specOCq_S,1,status,warndie)
          typeF = 2
          !
          ! only need expo and area from these corner spectrum file headers
          !
          call read_header_info(typeF,specOCq_S,specOCq_T,dummy_A,instD,& 
               obsidD,expidD,smodeD,frmtimeD,filtD,revlD,backTemp,areaTemp,& ! D for dummy
               expoTemp,elowD,ehighD,chansTemp,warndie) ! dummy for check, not used
          call read_pha(specOCq_T,chansO,1,chansO_IP,specOCq_JP,dummy_RP,specOCqU_RP)
          ! Get pattern for augment_pn usage 
          if (.not.hasAttribute(specOCq_S,'PATTERN')) then
             write(errstr,*)"Input file: ",trim(adjustl(specOCq_F))," has no PATTERN keyword"
             call error("noPATTERN",trim(adjustl(errstr)))
          else
             pattern = int32attribute(specOCq_S,'PATTERN')
          end if
          !
          ! sanity check if zero as above in FWC/FOV???!!!
          ! SS: If err = 0 then the file exists
          !
          expoOCq_tot(i) = expoTemp ! OLD: cor_exp_obs
          areaOCq_tot(i) = backTemp ! OLD: cor_area_obs
          !
          ! SS: Read in the OOT data
          ! Now do the CORNER OOT events 
          ! CORNER OOT by-quadrant spec OLDNEW: pnS002-3oc-oot.pi/pnS002-corootq3.pi
          !
          specOCOq_F = "pn" // trim(adjustl(expidO)) // "-corootq" // &
               trim(adjustl(cquad)) // ".pi"
          write(errstr,*)"Current quad OOT spectral file: ",trim(specOCOq_F)
          call message(AppMsg,Always)
          ! just for corner data:
          status = 0
          warndie  = .TRUE. ! change to false and not process          
          call check_open_set(specOCOq_F,specOCOq_S,1,status,warndie)
          ! only need back, expo, and area data from these spec file headers
          typeF = 2
          call read_header_info(typeF,specOCOq_S,specOCOq_T,dummy_A,instD,& 
               obsidD,expidD,smodeD,frmtimeD,filtD,revlD,backoTemp,areaoTemp,&
               expooTemp,elowD,ehighD,chansTemp,warndie) ! dummies for check, not used
          call read_pha(specOCOq_T,chansO,1,chansO_IP,specOCOq_JP,dummy_RP,specOCOqU_RP)
          ! Get pattern for augment_pn usage 
          if (.not.hasAttribute(specOCOq_S,'PATTERN')) then
             write(errstr,*)"Input file: ",trim(adjustl(specOCOq_F))," has no PATTERN keyword"
             call error("noPATTERN",trim(adjustl(errstr)))
          else
             patternOO = int32attribute(specOCOq_S,'PATTERN')
          end if
          ! or is this difference actually ok???!!!
          if (pattern .ne. patternOO) then ! or warning???!!!
             write(errstr,*)"PATTERN in ",trim(adjustl(specOCq_F)),&
                  ' must be the same as in OOT counterpart'
             call error("PATTnePATTOO",trim(adjustl(errstr)))
          endif
          !
          ! SS: Make the corner OOT corrected spectrum and get the hardness and 
          !  count rate parameters for the augmentation selection.  Note, these 
          !  spectra are not normalized (units are counts).
          !
          ratecorq    = 0. ! OLD/NEW: rate
          ! soft = 0. ! OLD: soft, never used!!!
          hardratio   = 0. ! OLD/NEW: hard/hardratio
          cntscorqH   = 0. ! OLD/NEW: high/cntscorqH
          cntscorqL   = 0. ! OLD/NEW: low/cntscorqL
          cntscorqV   = 0. ! OLD/NEW: hih/cntscorqV          
          hardratioU  = 0. ! OLD/NEW: sighard/hardratioU
          !
          ! SS: Calculate the OOT subtracted observation corner spectrum and uncertainty
          !
          do ichan=1,chansO
             !write(40,*)specOCq_JP(ichan-1),specOCqU_RP(ichan-1)
             specOCq(ichan) = real(specOCq_JP(ichan-1)) -  &     ! OLD: spec
                  ootsca*(real(specOCOq_JP(ichan-1)) * &         ! OLD: spec_oot
                  (expoTemp/expooTemp)*(backTemp/backoTemp))     ! OLD: expo/back
             specOCqU(ichan) = sqrt(specOCqU_RP(ichan-1)**2 +  & ! OLD: uncr
                  (ootsca*ootsca*specOCOqU_RP(ichan-1)*  &       ! OLD: uncr_oot
                  (expoTemp/expooTemp)*(backTemp/backoTemp))) ! OLD: expo/back/expo_oot/back_oot
             !write(41,*)specOCq(ichan),specOCqU(ichan)
             !write(42,*)specOCOq_JP(ichan-1),specOCOqU_RP(ichan-1)
             !write(43,*)ootsca,expoTemp,expooTemp,backTemp,backoTemp
             ! looks good!!
          enddo
          !
          !  SS: Find the number of counts in the observed spectrum and get the 
          !      count rate
          !  BHP: soft and low were used interchangeably! soft is never actually used 
          ratecorq = 0. ! OLD: rate
          do ichan=plc_all(1),plc_all(2)
             ratecorq = ratecorq + specOCq(ichan) ! OLD: rate + spec(ichan)
          enddo
          write(errstr,2998) ratecorq, expoOCq_tot(i) ! look good!!!
2998      format(' Counts (0.4-12.0 keV): ', f9.1,'  Exposure: ', f9.1)
          call message(AppMsg,Always)
          !
          !  SS: Find the hard and soft count rates and hardness ratio for the 
          !  observation corner data
          !
          do ichan=plc_hih(1),plc_hih(2)
             cntscorqV = cntscorqV + specOCq(ichan) ! OLD: hih/spec
          enddo
          do ichan=plc_hi(1),plc_hi(2)
             cntscorqH = cntscorqH + specOCq(ichan) ! OLD: high/spec
          enddo
          do ichan=plc_lo(1),plc_lo(2)
             cntscorqL = cntscorqL + specOCq(ichan) ! OLD: low/spec
          enddo
          !print*,'cntsVHL = ',cntscorqV, cntscorqH, cntscorqL !OLD: hih,high,low
          ! good!!!
          !
          if (cntscorqL .ne. 0) then 
             hardratio = cntscorqH/cntscorqL ! OLD: hard = high/low, check for divide by 0.
             hardratioU = sqrt((cntscorqH/(cntscorqL**2)) + & ! OLD: sighard
                  ((cntscorqH**2)*cntscorqL)/cntscorqL**4)    ! OLD: high/low
          else 
             call error("softEQzero","soft band rate = 0, gives infinite hardness ratio!")
          endif
          !
          ratecorq = cntscorqL + cntscorqH ! OLD: rate/low/high overwrites the above 
          ratecorqU = sqrt(ratecorq)       ! OLD: sigrate

          write(errstr,*)'low, high, hard, sighard rate/expo, sigrate/expo, expo'
          call message(AppMsg,Always) ! look good!!!
          write(errstr,2999) cntscorqL,cntscorqH,hardratio,hardratioU,  & 
               ratecorq/expoOCq_tot(i),ratecorqU/expoOCq_tot(i),expoOCq_tot(i)
2999      format(2f8.1,2f8.3,2f8.5,f8.1)
          call message(AppMsg,Always)

          !
          ! SS: Normalize the count rate, etc., /s/arcmin^2
          !
          ratecorq = amsca*ratecorq/ &         ! OLD: rate/cor_exp_obs/cor_area_obs
               expoOCq_tot(i)/areaOCq_tot(1)   ! OLD: cor_exp_obs/cor_area_obs
          ratecorqU = amsca*ratecorqU/ &       ! OLD: sigrate/
               expoOCq_tot(i)/areaOCq_tot(i)   ! OLD: cor_exp_obs/cor_area_obs
          ratecorqL = amsca*cntscorqL/ &       ! OLD: soft_rate/low/cor_exp_obs
               expoOCq_tot(i)/areaOCq_tot(i)   ! OLD: cor_exp_obs/cor_area_obs
          ratecorqH = amsca*cntscorqH/ &       ! OLD: hard_rate/high/cor_exp_obs 
               expoOCq_tot(i)/areaOCq_tot(i)   ! OLD: cor_exp_obs/cor_area_obs
          ratecorqV = amsca*cntscorqV/ &       ! OLD: hihrat/hih
               expoOCq_tot(i)/areaOCq_tot(i)   ! OLD: cor_exp_obs/cor_area_obs
          !
          ! SS: Set the search values for the augmentation spectra
          ! BHP: was resetting c_val(1:3), now 
          !
          ! c_val(1) = ratecorq
          ! c_val(2) = hardratio
          ! c_val(3) = revlD
          !
          write(errstr,3021) ratecorq, ratecorqU ! OLD: rate (c_val(1))/sigrate
3021      format(' Normalized Count Rate: ',f8.6,' +/-',f8.6)
          call message(AppMsg,Always)
          write(errstr,3022) hardratio, hardratioU ! OLD: hard (c_val(2))/sighard
3022      format(' Hardness: ',f8.3,' +/-',f8.3)
          call message(AppMsg,Always)
          write(errstr,3023) cntscorqL, cntscorqH, cntscorqV ! OLD: low,high,hih
3023      format(' Counts: ', 3f8.1)
          call message(AppMsg,Always)
          write(errstr,3123) ratecorqL, ratecorqH, ratecorqV ! OLD: soft_rate/hard_rate/hihrat
3123      format(' Count Rates: ', 3f8.5)
          call message(AppMsg,Always)
          write(errstr,*)" Revolution = ",revlD ! OLD: c_val(3)
          call message(AppMsg,Noisy)
          ! above is good!!!
          !          
          !  SS: Augment the observed corner spectrum - normalized /s/arcmin^2
          !      This is the old way of doing things.
          !
          write(errstr,*)'Augment the corner spectrum' 
          call message(AppMsg,Always)
          write(errstr,*)'Corner rate     = ',ratecorq   ! c_val(1)
          call message(AppMsg,Always)
          write(errstr,*)'Corner hardness = ',hardratio  ! c_val(2)
          call message(AppMsg,Always)
          write(errstr,*)'Revolution      = ',revlD      ! c_val(3)
          call message(AppMsg,Always)
          write(errstr,*)'aug_scal        = ',aug_scal   ! (/0.00005, 0.2, 300.0/)
          call message(AppMsg,Always)
          write(errstr,*)'plc_lo/plc_hi   = ',plc_lo,plc_hi
          call message(AppMsg,Noisy)
          !
          write(cquad,'(I1)')i
          write(errstr,*)'Before augmenting spectrum for CCD: '// cquad
          call message(AppMsg,Noisy) ! change to Noisy???!!!

          aug_spec = 0.
          aug_expo = 0.
          !
          ! OLD:  call aug_get_pnexp(db_file,c_val,aug_scal,aug_time,&
          !       pflag,aug_spec,aug_expo,i,obsid,prefix,mode,plc_lo,plc_hi)
          call augment_pn(QPB_S,ratecorq,hardratio,revlD,ootsca,aug_scal,&
               aug_time,aug_spec,aug_expo,i,chansO,obsidO,expidO,smodeD,&
               plc_lo,plc_hi,lunratehard,lunaugspec,lunraterev,lunratehrev,&
               firsttime,withqdp,pattern)
          write(errstr,*)'c_val(1) = ',ratecorq
          call message(AppMsg,Noisy) ! change to Noisy???!!!
          write(errstr,*)'c_val(2) = ',hardratio
          call message(AppMsg,Noisy) ! change to Noisy???!!!
          write(errstr,*)'c_val(3) = ',revlD
          call message(AppMsg,Noisy) ! change to Noisy???!!!
          write(errstr,*)'aug_scal = ',aug_scal
          call message(AppMsg,Always) ! change to Noisy???!!!
          write(errstr,*)'aug_time = ',aug_time
          call message(AppMsg,Always) ! change to Noisy???!!!
          write(errstr,*)'aug_expo = ',aug_expo
          firsttime = .FALSE. ! retain plot luns for all quad writes
          write(errstr,*)'After augmenting spectrum for CCD: '// cquad
          call message(AppMsg,Noisy) ! change to Noisy???!!!
          !
          ! Apply the augmented spectrum
          !
          do ichan=1,chansO
             specOCqU(ichan) = sqrt(specOCqU(ichan)**2 + & ! OLD:uncr
                  aug_expo * (areaOCq_tot(i)/amsca)* & ! OLD: aug_expo/cor_area_obs
                  aug_spec(ichan))/ &                  ! OLD: aug_spec
                  (areaOCq_tot(i)/amsca)/ &            ! OLD: cor_area_obs
                  (expoOCq_tot(i)+aug_expo)            ! OLD: cor_expo_obs/expocorQ
             specTemp(ichan) = (specOCq(ichan) + &     ! OLD: tspec/spec 
                  aug_expo*(areaOCq_tot(i)/amsca)* & ! OLD: aug_expo/cor_area_obs
                  aug_spec(ichan))/ &          ! OLD: aug_spec
                  (areaOCq_tot(i)/amsca)/ &    ! OLD: cor_area_obs/areacorQ
                  (expoOCq_tot(i)+aug_expo)    ! OLD: cor_expo_obs/aug_expo 
             if(specTemp(ichan) .lt. 0.0) then ! tspec
                specTemp(ichan) = 0.0          ! tspec
                specOCqU(ichan) = 0.0          ! uncr
             endif ! looks good!!!
             !write(44,*)specOCqU(ichan),specTemp(ichan)
          end do
          !
          do ichan=1,chansO
             specOCq(ichan) = specTemp(ichan) ! OLD/NEW: spec/specOCq (obs+cor+quad)
             !write(45,*)specOCq(ichan) looks good!!!
          end do
          !
          write(errstr,3025) expoOCq_tot(i),aug_expo
3025      format(' Original and augmented ','exposure times: ',f8.1," ",f11.1)
          call message(AppMsg,Always)
          !
          ! SS: Add the observation exposure to the...missing text here???!!!
          !     augment this temp quad expo from its original value???!!!
          !
          expo = expoOCq_tot(i) + aug_expo ! OLD: cor_exp_obs/aug_expo
          expoOCq_tot(i) =  expo 
          !
          ! SS: Test the augmentation of the background spectrum or compare the 
          !     canned spectrum to the observation.
          !     Find and print out some count rates
          !     Soft, Hard, and Very Hard bands
          ! BHP: zeroed out from above calculations???!!!
          !
          cntscorqL = 0.0 ! OLD: low
          cntscorqH = 0.0 ! OLD: high
          cntscorqV = 0.0 ! OLD: hih
          do ichan=plc_lo(1),plc_lo(2)
             cntscorqL = cntscorqL + specOCq(ichan) ! low = low+spec(ichan)
          enddo
          do ichan=plc_hi(1),plc_hi(2)
             cntscorqH = cntscorqH + specOCq(ichan) ! high = high+spec(ichan)
          enddo
          do ichan=plc_hih(1),plc_hih(2)
             cntscorqV = cntscorqV + specOCq(ichan) ! hih = hih+spec(ichan)
          enddo
          write(errstr,*)'low, high, hih = ',cntscorqL,cntscorqH,cntscorqV
          call message(AppMsg,Always)
          !
          ! SS:The augmented count rate is the sum of the soft and 
          !    hard band count rates
          !
          aug_rate = cntscorqL + cntscorqH ! OLD: aug_rate = low+high
          !print*,'aug_rate = ',aug_rate looks good!!!

          !  The hardness ratio is the ratio of the hard to 
          !  soft bands
          ! change these to Verbose for release???!!!
          ! aren't some of these duplicative???!!!
          write(errstr,*)'Comparisons after augumentation'
          call message(AppMsg,Always)
          write(errstr,*)'Comparison of the count rate' ! c_val(1) = ratecorq
          call message(AppMsg,Always) ! OLD: c_val(1), aug_rate, aug_rate/c_val(1)
          write(errstr,3026)ratecorq,aug_rate,aug_rate/ratecorq
          call message(AppMsg,Always)
3026      format(' Before, After, Ratio: ',2f15.6,f15.6)
          write(errstr,*)'Comparison of the very hard band'
          call message(AppMsg,Always) ! OLD: hihrat,hih,hih/hihrat
          write(errstr,3026)ratecorqV,cntscorqV,cntscorqV/ratecorqV
          call message(AppMsg,Always)
          write(errstr,*)'Comparison of the hard band'
          call message(AppMsg,Always) ! OLD: hard_rate,high,high/hard_rate
          write(errstr,3026)ratecorqH,cntscorqH,cntscorqH/ratecorqH
          call message(AppMsg,Always)
          write(errstr,*)'Comparison of the soft band'
          call message(AppMsg,Always) ! OLD: soft_rate,low,low/soft_rate
          write(errstr,3026)ratecorqL,cntscorqL,cntscorqL/ratecorqL
          call message(AppMsg,Always)
          write(errstr,*)'Comparison of the hardness ratio'
          ! OLD: c_val(2), 
          write(errstr,3026)hardratio,cntscorqH/cntscorqL,&
               (cntscorqH/cntscorqL)/hardratio
          call message(AppMsg,Always)
          write(errstr,*)'Augmented low, high, and hardness'
          call message(AppMsg,Always) ! OLD: low, high, high/low
          write(errstr,3027) cntscorqL,cntscorqH,cntscorqH/cntscorqL
          call message(AppMsg,Always)
3027      format(' ',2f8.5,f8.3)
          ! looks good!!!
          !
          ! SS: Scale by count rate and then test the scaling of the 
          !     background spectrum
          ! BHP: this had an "if (norm.eq.1)" block but it was hardcoded to 1, why???!!!
          ! c_val(1) == ratecorq
          write(errstr,*)'Scale by count rate and then test the scaling of the ' // &
               'background spectrum'
          call message(AppMsg,Always)
          do ichan=1,chansO
             specOCq(ichan)  = specOCq(ichan)*ratecorq/aug_rate ! OLD:spec/c_val(1)/aug_rate
             specOCqU(ichan) = specOCqU(ichan)*ratecorq/aug_rate ! OLD:uncr/c_val(1)/aug_rate
             !             write(46,*)specOCq(ichan),specOCqU(ichan)
          enddo ! look good!!!

          cntscorqL = 0.0 ! OLD: low
          cntscorqH = 0.0 ! OLD: high
          cntscorqV = 0.0 ! OLD: hih
          !
          do ichan=plc_lo(1),plc_lo(2)
             cntscorqL = cntscorqL + specOCq(ichan) ! OLD: low/low/spec
          enddo
          do ichan=plc_hi(1),plc_hi(2)
             cntscorqH = cntscorqH + specOCq(ichan) ! OLD: high/high/spec
          enddo
          do ichan=plc_hih(1),plc_hih(2)
             cntscorqV = cntscorqV + specOCq(ichan) ! OLD: hih/hih/spec
          enddo
          ! change to Verbose for release???!!!
          write(errstr,*) 'Comparisons after scaling'
          call message(AppMsg,Always)
          write(errstr,*) 'Comparison of the count rate'
          call message(AppMsg,Always)
          write(errstr,3026)ratecorq,aug_rate,aug_rate/ratecorq
          call message(AppMsg,Always)
          write(errstr,*) 'Comparison of the very hard band'
          call message(AppMsg,Always)
          write(errstr,3026)ratecorqV,cntscorqV,cntscorqV/ratecorqV
          call message(AppMsg,Always)
          write(errstr,*) 'Comparison of the hard band'
          call message(AppMsg,Always)
          write(errstr,3026)ratecorqH,cntscorqH,cntscorqH/ratecorqH
          call message(AppMsg,Always)
          write(errstr,*) 'Comparison of the soft band'
          call message(AppMsg,Always)
          write(errstr,3026)ratecorqL,cntscorqL,cntscorqL/ratecorqL
          call message(AppMsg,Always)
          write(errstr,*) 'Comparison of the hardness ratio'
          call message(AppMsg,Always)
          write(errstr,3026)hardratio,cntscorqH/cntscorqL,(cntscorqH/cntscorqL)/hardratio
          call message(AppMsg,Always)
          write(errstr,*) 'Augmented low, high, and hardness'
          call message(AppMsg,Always)
          write(errstr,3027)cntscorqL,cntscorqH,cntscorqH/cntscorqL
          call message(AppMsg,Always)
          !
          ! SS: The spectrum is normalized to per unit area (arcmin^2) per second
          !
          do ichan=1,chansO
             specOCq_tot(ichan,i)  = specOCq(ichan)  ! OLD oc_spec = spec
             specOCqU_tot(ichan,i) = specOCqU(ichan) ! OLD oc_uncr = uncr
             !             write(15,*)i,ichan,specOCq_tot(ichan,i),specOCqU_tot(ichan,i)
             ! looks good!
          enddo

          ! OLD: cor_area_obs,amsca,cor_exp_obs
          write(errstr,3028) i, areaOCq_tot(i)/amsca,expoOCq_tot(i)
3028      format(' Quad:',i2,' Back:',f6.2,' Exposure:',f11.1)
          call message(AppMsg,Always)
       else 
          specOCq_tot    = 0.0   ! OLD: oc_spec, removed do loop
          specOCqU_tot   = 0.0   ! OLD: oc_uncr, removed do loop
          areaOCq_tot(i) = 0.0   ! OLD: cor_area_obs
          if(areaOFq_tot(i) .ne.  0.0) then ! OLD: reg_area_fwc
             specOFq_tot   = 0.0 ! OLD: reg_spec
             specOFqU_tot  = 0.0 ! OLD: reg_uncr
             areaOFq_tot(i) = 0. ! OLD: reg_area_fwc
             write(errstr,*)"Missing file: ",trim(adjustl(specOCOq_F))
             call error("PNBACK09",trim(adjustl(errstr)))
             ! OLD PN_BACK POSITION 09
          endif
       endif
       call release(specOCq_S)
       call release(specOCOq_S)
    end do ! end SECTION 4 loop through quads with the corner spectra
    ! deallocate here ???!!!
    write(errstr,*)'Backscale and augmented exposure from the ', &
         'corners of the observation data'
    call message(AppMsg,Always) ! change to Noisy???!!!
    do i=1,4
       if (quads(i)) then ! BHP: a boolean now
          write(errstr,3029) i, areaOCq_tot(i),  &
               areaOCq_tot(i)/amsca, expoOCq_tot(i)
3029      format(' Quadrant',i2,'  backscale=',f10.0, &
               ' backscale/amsca= ',f7.2,'  exposure =',f9.1)
          call message(AppMsg,Always) ! BHP: verbose maybe???!!!
       endif
    end do

    ! good to here!!!
    !
    ! SECTION 5: FWC CORNER data by-quad CFq/CFOq suffix
    !
    ! SS: The fc data are the FWC data from the corners
    !
    ! read in the by-quadrant FWC corner and corner oot data, create:
    ! specCFq(4096,4), only fill the 4 where quad=T
    ! specCFqU(4096,4), the uncertainties on specCFq
    ! areaCFq_tot and expoCFq_tot are r*4 x 4 elements, one for each quad
    !
    allocate(specCFq_tot(chansO,4),specCFqU_tot(chansO,4)) ! all quad's spec
    allocate(specCFq(chansO),specCFqU(chansO))            ! temp each quad's spec
    allocate(expoCFq_tot(4),areaCFq_tot(4))
    expoCFq_tot = 0.  ! OLD: cor_exp_fwc
    areaCFq_tot = 0.  ! OLD: cor_area_fwc
    specCFq  = 0.     ! OLD: spec(4096) temp in old 
    specCFqU = 0.     ! OLD: uncr(4096) temp in old
    specCFq_tot = 0.  ! OLD: fc_spec(4096,4)
    specCFqU_tot = 0. ! OLD: fc_uncr(4096,4)
    ! SS: print *, 'Process the FWC corner data'
    ! TYPE: CFq/CFOq
    ! FWC corner by-quad spectra
    do i=1,4
       if( quads(i) ) then ! BHP: boolean now
          write(cquad,'(I1)')i
          ! by-quadrant spec OLDNEW pnS002-3fc.pi/pnS002-fwccorq3.pi
          specCFq_F = "pn" // trim(adjustl(expidO)) // "-" // &
               "corfwcq" // trim(adjustl(cquad)) // ".pi"
          write(errstr,*)"Current FWC CORNER quad spectral file     : ",trim(specCFq_F)
          call message(AppMsg,Always) ! change to Noisy???!!!
          revlD     = 0
          backTemp  = 0.
          areaTemp  = 0.
          expoTemp  = 0.
          backoTemp = 0.
          areaoTemp = 0.
          expooTemp = 0.
          status    = 0
          warndie   = .TRUE. 
          call check_open_set(specCFq_F,specCFq_S,1,status,warndie)
          typeF = 2
          !
          ! only need back, expo, and area from these spec file headers
          !
          call read_header_info(typeF,specCFq_S,specCFq_T,dummy_A,instD,& ! D for dummy
               obsidD,expidD,smodeD,frmtimeD,filtD,revlD,backTemp,areaTemp,& 
               expoTemp,elowD,ehighD,chansTemp,warndie) ! dummy for check, not used
          !
          ! only need specCFq_JP, a fits 'J' column put in a ptr array
          !
          call read_pha(specCFq_T,chansO,1,chansO_IP,specCFq_JP,dummy_RP,specCFqU_RP)
          !
          areaCFq_tot(i) = backTemp ! OLD: cor_area_fwc/back
          expoCFq_tot(i) = expoTemp ! OLD: cor_exp_fwc/expo 
          !
          ! SS: Now do the OOT events
          ! 
          ! FWC oot by-quadrant spec OLDNEW: pnS002-3fc-oot.pi/pnS002-fwccorootq3.pi
          !
          specCFOq_F = "pn" // trim(adjustl(expidO)) // "-" // &
               "corfwcootq" // cquad // ".pi"
          write(errstr,*)"Current FWC CORNER quad OOT spectral file : ",trim(specCFOq_F)
          call message(AppMsg,Always)
          backoTemp = 0.
          areaoTemp = 0.
          expooTemp = 0.
          status    = 0
          warndie   = .TRUE. 
          call check_open_set(specCFOq_F,specCFOq_S,1,status,warndie)
          ! only need expo and area data from these spec file headers
          typeF = 2
          call read_header_info(typeF,specCFOq_S,specCFOq_T,dummy_A,instD,& ! D for dummy
               obsidD,expidD,smodeD,frmtimeD,filtD,revlD,backoTemp,areaoTemp,&
               expooTemp,elowD,ehighD,chansoTemp,warndie) ! dummy for check, not used
          ! only need specoot_JP, a fits 'J' column put in a ptr array
          call read_pha(specCFOq_T,chansO,1,chansO_IP,specCFOq_JP,dummy_RP,specCFOqU_RP)
          !
          ! SS: Subtract the OOT events from the spectrum and normalize the 
          !     result to counts/s/unit area (arcmin pixels).
          ! spec+fwc+corner+Uncertainty+total
          do ichan=1,chansO ! OLD/NEW: fc_spec/fc_uncr spec/specCFq_JP/specCFqoot_JP
             specCFqU_tot(ichan,i) = amsca*sqrt(real(specCFq_JP(ichan-1))+&! OLD: fc_uncr
                  ootsca*ootsca*real(specCFOq_JP(ichan-1))* & 
                  (backTemp/backoTemp)* & ! OLD: back/back_oot
                  (expoTemp/expooTemp))/backTemp/expoTemp ! OLD expo/expo_oot/back/expo
             specCFq_tot(ichan,i) = & ! OLD: fc_spec
                  amsca*(real(specCFq_JP(ichan-1)/backTemp/expoTemp - &  ! OLD: spec/back/expo
                  ootsca*real(specCFOq_JP(ichan-1))/backoTemp/expooTemp))! OLD: spec_oot/back/expo
             ! old channels start at 61, new at 31
             ! counts: new = 100xold for first 30 channels
             ! last 900 old channel counts = 0, not so for new 
             !write(55,*)i,ichan,specCFq_tot(ichan-1,i),specCFqU_tot(ichan-1,i)
          enddo
       else ! OLD: reg_area_fwc
          if ((areaOFq_tot(i) .ne. 0.0) .or. (areaOCq_tot(i) .ne. 0.0)) then
             write(errstr,*)"Missing file: ",trim(adjustl(specCFOq_F))
             call error("PNBACK11",trim(adjustl(errstr))) ! does this ever happen???!!!
          end if
          ! check PN-BACK POSITION 11 code block???!!!
       endif
       call release(specCFq_S)
       call release(specCFOq_S)
    end do ! end SECTION 5 do loop through quads i=1,4, of FWC CORNER spectra
    ! looks good!!!
    ! do deallocate here???!!!    
    write(errstr,*) 'Backscale and exposure from the corners of the FWC data:'
    call message(AppMsg,Always)
    do i=1,4
       if(quads(i)) then ! exposures are 30% higher in 'old' 
          write(errstr,*)'Quadrant ', i, ' backscale=',areaCFq_tot(i)/amsca, &
               ' exposure =', expoCFq_tot(i)
          call message(AppMsg,Always)
       endif
    end do
    !
    ! SECTION 6: SMOOTHING the Data
    ! SS: Set up for smoothing the data
    ! OLD: temp_top/temp_bot/temp_oc
    allocate (temp_top(chansO),temp_bot(chansO),temp_cor(chansO))
    allocate (specmask_tot(chansO,4),specrati_tot(chansO,4))
    specmask_tot = 0. ! OLD: reg_mask
    specrati_tot = 0. ! OLD: reg_rati
    temp_top  = 0.
    temp_bot  = 0.
    temp_cor  = 0.
    ! SS:  Create the ratio of region/corner for FWC data
    !print*,'plc_alk = ',plc_alk
    !print*,'plc_hard = ',plc_hard look good!!!
    do i=1,4 
       if((areaOFq_tot(i) .ne. 0.0) .and. (quads(i))) then ! OLD: reg_area_fwc
          !
          !  Set up for smoothing the data (new code for SAS-17 BHP)
          !
          do ichan=1,chansO
             if(ichan .lt. 61) then
                i1 = ichan
                i2 = ichan
             elseif(ichan .lt. 61+smo_fact) then ! smo_fact is hardcoded to 12
                i1 = 61
                i2 = ichan + (ichan - 61)
             else
                i1 = ichan - smo_fact
                i2 = ichan + smo_fact
             endif
             if(i1 .lt. 1) i1 = 1
             if(i2 .gt. chansO) i2 = chansO
             ! i1/i2 look good!!!
             !
             !  SS: Check to see if the channel (ichan) is in a bridge region,
             !  and if so ignore
             !
             if(( (ichan  .ge. plc_alk(1))   .and. &
                  (ichan  .le. plc_alk(2)))  .or. &
                  ((ichan .ge. plc_hard(1))  .and. &
                  (ichan  .le. plc_hard(2)))) then
                temp_top(ichan) = 0.0
                temp_bot(ichan) = 0.0
                temp_cor(ichan) = 0.0
                ! the above are identical old v new
             else 
                !
                !  SS: Exclude data if they are in a bridge region - note that 
                !  when the channel being smoothed is close to a bridge region
                !  that there will be one-sided smoothing but this should not
                !  be a significant effect as the values only show up as a ratio
                !  where the effect should be very similar for both the numberator
                !  and denominator.
                !
                if(i1 .lt. plc_alk(1)) then
                   if(i2 .gt. plc_alk(1)) i2 = plc_alk(1)
                elseif(i1 .lt. plc_hard(1)) then
                   if(i1 .lt. plc_alk(2)) then
                      i1 = plc_alk(2)
                   elseif(i2 .gt. plc_hard(1)) then
                      i2 = plc_hard(1)
                   endif
                else
                   if(i1 .lt. plc_hard(2)) i1 = plc_hard(2)
                endif
                ! the above i1,i2 are identical b/w old/new
                !
                !  SS: The code used to check to make sure that bins had non-zero data
                !  I don't see the reason as it would bias the data higher
                !
                t1 = 0.0
                t2 = 0.0
                t3 = 0.0
                it = 0
                do ii=i1,i2
                   t1 = t1 + specOFq_tot(ii,i)  ! OLD: reg_spec fov fwc
                   t2 = t2 + specCFq_tot(ii,i)  ! OLD: fc_spec  cor fwc
                   t3 = t3 + specOCq_tot(ii,i)  ! OLD: oc_spec  fov cor
                   it = it + 1
                enddo
                if(it .gt. 0) then 
                   temp_top(ichan) = t1/it
                   temp_bot(ichan) = t2/it
                   temp_cor(ichan) = t3/it
                else
                   temp_top(ichan) = 0.0
                   temp_bot(ichan) = 0.0
                   temp_cor(ichan) = 0.0
                endif
             endif
             !
             !write(59,*)i,ichan,temp_top(ichan),temp_bot(ichan),temp_cor(ichan)
          enddo
          do ichan=1,chansO
             if(temp_bot(ichan) .ne. 0) then
                specmask_tot(ichan,i) = 1.0 ! OLD: reg_mask
                specrati_tot(ichan,i) = &   ! OLD: reg_rati
                     temp_top(ichan)/temp_bot(ichan) 
             else 
                specmask_tot(ichan,i) = 0.0 ! OLD: reg_mask
                specrati_tot(ichan,i) = 0.0 ! OLD: reg_rati
             endif
             !  write(43,*)i,ichan,specmask_tot(ichan,i),specrati_tot(ichan,i)
          enddo
       endif
    enddo ! end SECTION 6 smoothing
    ! these below look pretty good except high counts and starting at 31
    ! ff starts at 61, OF starts at 31
    ! fc starts at 31, CF starts at 31
    ! oc starts at 61, OC starts at 31
    ! do i=1,4
    !    do ii=1,4096
    !       write(60,*)i,ii,specOFq_tot(ii,i)
    !       write(61,*)i,ii,specCFq_tot(ii,i)
    !       write(62,*)i,ii,specOCq_tot(ii,i)
    !    enddo
    ! end do
    do i=1,chansO
       write(errstr,*)'TEMP_TOP = ',temp_top(i)
       call message(AppMsg,Noisy)
       write(errstr,*)'TEMP_BOT = ',temp_bot(i)
       call message(AppMsg,Noisy)
       !       write(63,*)temp_top(i),temp_bot(i),temp_cor(i)
    enddo
    !
    ! SS: Print out some diagnostic information
    ! SECTION 7
    !    print*,'Section 7'
    write(errstr,*) '  '
    write(errstr,*) 'Backscale for the FWC corner and FWC region data'
    call message(AppMsg,Always)
    do i=1,4 ! OLD: cor_area_fwc, reg_area_fwc
       write(errstr,*)'Quadrant ', i, ' backscale=', &
            areaCFq_tot(i)/amsca,' backscale =', areaOFq_tot(i)/amsca
       call message(AppMsg,Always) ! change to Verbose???!!!
    enddo
    !
    t1 = 0.0
    t2 = 0.0
    do i=1,4 
       t1 = t1 + areaCFq_tot(i) ! OLD: cor_area_fwc
       t2 = t2 + areaOFq_tot(i) ! OLD: reg_area_fwc
    enddo
    write(errstr,*)'t1 t2 = ',t1,t2
    call message(AppMsg,Always)
    write(errstr,*) 'FWC Region Area, FWC Corner Area, Corner Fraction: '
    call message(AppMsg,Always)
    write(errstr,*) t1/amsca, t2/amsca, t1/(t1 + t2)
    call message(AppMsg,Always)
    ! end SECTION 7
    !
    !  SS: Create the masks to remove strong instrumental lines
    !  Remove the Al line from all spectra
    ! SECTION 8
    !    print*,'Section 8'
    do ichan=plc_alk(1),plc_alk(2)
       do i=1,4
          maskOC_tot(ichan,i) = 0. ! OLD: oc_mask
       enddo
    enddo
    do ichan=plc_hard(1),plc_hard(2)
       do i=1,4
          maskOC_tot(ichan,i) = 0. ! OLD: oc_mask
       enddo
    enddo
    ! looks good!!!
    ! end SECTION 8
    !
    ! SS: Sum the background over all active chips and make an image
    !
    ! SECTION 9

    if (withqdp) then
       ! OLDNEW: pnS001-quad-spec.qdp/pnS001-quadspec.qdp
       call open_ascii_file(qdpquadspec_F,lunquadspec)
       !       print*,'Section 9 ',lunquadspec
       write(lunquadspec,*) 'viewport 0.2 0.2 0.9 0.85'
       write(lunquadspec,*) 'read serr 2 3 4 5'
       write(lunquadspec,*) 'font roman'
       write(lunquadspec,*) 'cs 1.5'
       ! create title for qdp plot
       write(title,*) trim(adjustl(obsidO))," -- PN",trim(adjustl(expidO))
       write(lunquadspec,*) 'la t ', title
       write(lunquadspec,*) 'la f'
       write(lunquadspec,*) 'log y'
       write(lunquadspec,*) 'la x Energy (keV)'
       write(lunquadspec,*) 'la y Scaled Counts'
       write(lunquadspec,*) 'la 1 p 1 100 cs 0.9 just left ', &
            '"Quad 1 (red), Quad 2 (green, scaled by 0.1)"'
       write(lunquadspec,*) 'la 2 p 1 40 cs 0.9 just left ', &
            '"Quad 3 (dk blue, scaled by 0.01), Quad 4 ', &
            '(lt blue, scaled by 0.001)"'
       write(lunquadspec,*) 'r x 0 15.0'
    end if
    ! end SECTION 9
    allocate(specFinal(chansO),specFinalU(chansO),maskFinal(chansO))
    allocate(image(xGood,yGood),imageFinal(xGood,yGood))
    specFinal  = 0. ! 4096
    specFinalU = 0. ! 4096
    image = 0.      ! 780,780 OLD: image
    imageFinal = 0. ! 780,780 OLD: outim
    tempsp  = 0.
    tempunc = 0.
    ! SECTION 10 create final image and spectrum
    !    print*,'Section 10'
    do i=1,4
    enddo
    do i=1,4
       !  SS: Loop over the four quadrants
!       write(46,*)" sum of quad: ",i," = ",areaOFq_tot(i)," quad = ",quads(i)," quads = ",quads
       if((areaOFq_tot(i) .ne. 0) .and. quads(i)) then ! OLD: reg_area_fwc
          !  SS: Loop over the spectral energy bins and add to the count spectrum
          !  and the spectrum uncertainty for the entire region.
          do ichan=1,chansO
             ! the old stops at channel 3213, new is all 4096???!!!
             if((specmask_tot(ichan,i) .ne. 0.0) .and. & ! OLD: reg_mask
                  (maskOC_tot(ichan,i) .ne. 0.0)) then   ! OLD: oc_mask
                tempsp = specOCq_tot(ichan,i)* &         ! OLD: oc_spec
                     specrati_tot(ichan,i)* &            ! OLD: reg_rati
                     expoOq_tot(i)* &                    ! OLD: reg_exp_obs
                     areaOq_tot(i)/amsca                 ! OLD: reg_area_obs
                tempunc = specOCqU_tot(ichan,i)* &       ! OLD: oc_uncr
                     specrati_tot(ichan,i)* &            ! OLD: reg_rati
                     expoOq_tot(i)* &                    ! OLD: reg_exp_obs
                     areaOq_tot(i)/amsca                 ! OLD: reg_area_obs
                ! OLD/NEW: mask/maskFinal reg_area_fwc/areaOFq_tot
                ! lots of high points in new chans below 61, above look
                ! good but old channels drop out at 3213 again of course
                specFinal(ichan)  = specFinal(ichan)  + tempsp     ! OLD: spec
                specFinalU(ichan) = specFinalU(ichan) + tempunc**2 ! OLD: uncr
                maskFinal(ichan)  = maskFinal(ichan)  + areaOFq_tot(i)
!                if (i .gt. 1) then 
!                   write(45,*)i,ichan,specFinal(ichan),specFinalU(ichan),maskFinal(ichan)
!                endif
                if (withqdp) then 
                   if(i .eq. 1) then
                      write(lunquadspec,*) energy(ichan), tempsp, &
                           tempunc, '  0.0 0.0 0.0 0.0 0.0 0.0'
                   elseif(i .eq. 2) then
                      write(lunquadspec,*) energy(ichan), '  0.0 0.0',  &
                           0.1*tempsp, 0.1*tempunc,'  0.0 0.0 0.0 0.0'
                   elseif(i .eq. 3) then
                      write(lunquadspec,*) energy(ichan), '  0.0 0.0 0.0 0.0', & 
                           0.01*tempsp, 0.01*tempunc, '  0.0 0.0'
                   elseif(i .eq. 4) then
                      write(lunquadspec,*) energy(ichan), '  0.0 0.0 0.0 0.0', &
                           ' 0.0 0.0', 0.001*tempsp, 0.001*tempunc
                   endif
                endif
             endif         ! or else what???!!!
          enddo
          cntscorband = 0. ! OLD: tmodcounts corner counts in the band 
          tot1 = 0.0
          tot2 = 0.0
          !
          ! SS: Get the number of counts in the band
          !
          ! BHP: tot
          do ichan=band(1),band(2)
             cntscorband = cntscorband + specOCq_tot(ichan,i)* & ! OLD: tmodcounts/oc_spec
                  specrati_tot(ichan,i)*expoOq_tot(i)* &         ! OLD: reg_rati/reg_exp_obs
                  areaOq_tot(i)/amsca                            ! OLD: reg_area_obs
             tot1 = tot1 + specOCq_tot(ichan,i)* &               ! OLD: oc_spec
                  specrati_tot(ichan,i)*expoOq_tot(i)* &         ! OLD: reg_rati/reg_exp_obs
                  areaOq_tot(i)/amsca                            ! OLD: reg_area_obs
             tot2 = tot2 + specrati_tot(ichan,i)                 ! OLD: reg_rati
          enddo
          ! change to Verbose for release???!!!
          write(errstr,*) 'Chip Band-Counts Ratio  FWC-Backscale ', &
               'Obs-Backscale Exposure:'
          call message(AppMsg,Always) ! OLD:  reg_area_fwc/reg_area_obs/reg_exp_obs
          write(errstr,*)i,tot1,tot2,areaOFq_tot(i)/amsca,areaOq_tot(i)/amsca,expoOq_tot(i)
          call message(AppMsg,Always)
          !print*,i,' tmodcounts = ',cntscorband ! old=4212 new=36245???!!!
          !
          ! SS: Read in the fwc image
          ! OLDNEW: pnS001-im1-350-1100.fits/pnS003-fwcimq1det-350-1100.fits	
          ! BHP: put something here where these come from (pnspectra)
          ! get the names locked down???!!!
          if (withimg) then
             write(cquad,'(I1)') i ! OLD: im_name
             inFWC_F  = "pn" // trim(adjustl(expidO)) // "-fwcimq" // cquad // "det-" &
                  // trim(adjustl(celow)) // "-" // trim(adjustl(cehigh)) // ".fits"
             status = 0
             warndie = .TRUE. ! croak if not found or unreadable
             call check_open_set(inFWC_F,inFWC_S,1,status,warndie) ! goflib/gofio_mod
             ! extract image array and coordinate keys (these are in gofio/goflib also)
             !       call read_header_info(type,inIMG_S,inIMG_T,inIMG_A,inst,obsid,expid,&
             !            smode,frmt,filt,revl,back,area,expo,el,eh,nR,warndie)
             call read_2Darray(inFWC_S,inFWC_A,aDR32,aDR64,aDI16,inFWC_JP,xSize,ySize,typeA)
             !       call coord_keys (inIMG_S, ra_pnt, dec_pnt, pa_pnt, crval, cdelt, &
             !            cunit, ctype, crpix, crota, radecsys, warndie)
             ! check this???!!!
             ! OLDNEW: pnS003-im4-400-750-oot.fits/pnS003-fwcimootq4det-400-750.fits
             ! OLD: im_name_oot
             inFWCO_F = "pn" // trim(adjustl(expidO)) // "-fwcimootq" // cquad // &
                  "det-" // trim(adjustl(celow)) // "-" // trim(adjustl(cehigh)) // ".fits"
             status = 0
             warndie = .TRUE. ! croak if not found or unreadable
             call check_open_set(inFWCO_F,inFWCO_S,1,status,warndie) ! goflib/gofio_mod
             type = 3 ! image
             ! extract image array and coordinate keys (these are in gofio/goflib also)
             !       call read_header_info(type,inIMG_S,inIMG_T,inIMG_A,inst,obsid,expid,&
             !            smode,frmt,filt,revl,back,area,expo,el,eh,nR,warndie)
             call read_2Darray(inFWCO_S,inFWCO_A,aDR32,aDR64,aDI16,inFWCO_JP,xSize,ySize,typeA) 
             !       call coord_keys (inIMG_S, ra_pnt, dec_pnt, pa_pnt, crval, cdelt, &
             !            cunit, ctype, crpix, crota, radecsys, warndie)
             !
             ! SS: Check for bright pixels
             ! BHP: here the sums are new = 10xold
             !print*,'PPPPP sum = ',sum(inFWC_JP)
             !print*,'PPPPP sum = ',sum(inFWCO_JP)
             tot     = 0. ! OLD: tot
             totn    = 0. ! OLD: ntot
             avgNon0 = 0. ! OLD: avtot
             do j=0,xSize-1     ! 780
                do jj=0,ySize-1 ! 780
                   if(inFWC_JP(j,jj) .gt. 0) then                 ! OLD: image
                      image(j+1,jj+1) = real(inFWC_JP(j,jj)) - &  ! OLD: image
                           ootsca * real(inFWCO_JP(j,jj))         ! OLD: image_oot
                      tot = tot + real(inFWC_JP(j,jj))            ! OLD: image
                      totn = totn + 1.0
                   endif
                enddo
             enddo
             ! BHP: here, avgNon0 new = 3xold
             avgNon0 = tot/totn ! OLD: avtot
             write(errstr,*)'Average non-zero value of FWC image: ', avgNon0
             call message(AppMsg,Always)
             !
             ! SS: Reset bright pixels (5x average) to the average value
             !
             tot = 5.0*avgNon0
             do j=1,xSize
                do jj=1,ySize
                   if(image(j,jj) .gt. tot) then ! OLD: image
                      image(j,jj) = avgNon0      ! OLD: image
                   endif
                enddo
             enddo
             !
             tot = 0.         ! OLD: tot
             tot = sum(image) ! no double do loop needed, just use sum
             ! change to Verbose for release???!!!
             write(errstr,*) 'FWC image scaling for chip: '
             call message(AppMsg,Always)
             write(errstr,*) 'Chip Obs-Corner-Band-Counts ', &
                  'FWC-Corner-Band-Counts Ratio:'
             call message(AppMsg,Always)
             write(errstr,*) i, cntscorband, tot, cntscorband/tot
             call message(AppMsg,Always)
             do j=1,xSize      ! 780
                do jj=1,ySize  ! 780 OLD: outim/tmodcounts below
                   imageFinal(j,jj) = imageFinal(j,jj) + image(j,jj)*cntscorband/tot
                   ! write(46,*)j,jj,imageFinal(j,jj) looks good!!!
                enddo
             enddo
          endif
       endif
       if (withqdp) then
          write(lunquadspec,*) 'no no no'
       endif
    end do ! end SECTION 10 loop through the quads

    if (withqdp) then
       close(lunratehard)
       close(lunaugspec)
       close(lunraterev)
       close(lunratehrev)
       close(lunquadspec)
    end if
    !     SS: Add in the OOT events for CSTAT background spectrum
    !     SECTION 11
    !    print*,'Section 11'
    if(chisqfl) then
       specFinalU = sqrt(specFinalU)!  OLD/NEW: uncr/specFinalQU is this right???
    else
       do ichan=1,chansO ! OLD: spec = spec + ootspec 
          specFinal(ichan) = specFinal(ichan) + ootspec(ichan) ! OLD/NEW: spec/specFinal
          specFinalU(ichan) = sqrt(specFinalU(ichan) + ootspec(ichan))!  uncr/ootspec
       enddo
    endif
    ! old and new start at channel 31, new counts are 10xold 31-61
    ! channels 3213-4096 in new = 85xold ???!!!
    ! do i=1,chansO these seem ok, but channels 258-329 are new = 0.1xold
    !    write(47,*)i,specFinal(i),specFinalU(i)
    ! end do
    !     change to Verbose for release??? wasn't this printed earlier???
    write(errstr,*)'Exposure for the FWC and observation data'
    call message(AppMsg,Always)
    do i=1,4
       if(quads(i)) then
          write(errstr,*)'Quadrant ', i, ' exposure=', expoOFq_tot(i), &
               ' exposure =', expoOq_tot(i)
          call message(AppMsg,Always)
       endif
    enddo
    !    end SECTION 11

    !    SS: Search for and eliminate hot pixels
    !    SECTION 12
    !    print*,'Section 12'
    if(withimg) then
       if(ipix .eq. 1) then
          tot1 = 0.0
          tot2 = 0.0
          do i=1,xSize
             do ii=1,ySize
                if(imageFinal(i,ii) .gt. 0.0) then
                   tot1 = tot1 + imageFinal(i,ii)
                   tot2 = tot2 + 1.0
                endif
             enddo
          enddo
          tot1 = 2.0*tot1/tot2
          do i=1,xSize
             do ii=1,ySize
                if(imageFinal(i,ii) .gt. tot1) then
                   imageFinal(i,ii) = tot1
                endif
             enddo
          enddo
          tot1 = 0.0
          tot2 = 0.0
       endif
    endif  !end SECTION 12
    ! looks ok...except sum new = 10xold
    do i=1,780
       do ii=1,780
          !          if (imageFinal(i,ii) .ne. 0.) then
          !             write(48,*)i,ii,imageFinal(i,ii)
          !          endif
       end do
    end do
    write(errstr,*)' Sum of counts in final image = ',sum(imageFinal)

    ! SS: Write out some more diagnostic information
    ! SECTION 13
    !    print*,'Section 13'
    if (withqdp) then
       ! OLD: pnS001-back-spec.qdp/pnS001-backspec.qdp
       ! create title for qdp plot
       call open_ascii_file(qdpbackspec_F,lunbackspec)
       write(lunbackspec,*) 'viewport 0.2 0.2 0.9 0.85'
       write(lunbackspec,*) 'read serr 2'
       write(lunbackspec,*) 'font roman'
       write(lunbackspec,*) 'cs 1.8'
       write(lunbackspec,*) 'line off 3'
       write(lunbackspec,*) 'col off 3'
       write(title,*) trim(adjustl(obsidO))," -- PN",trim(adjustl(expidO))
       write(lunbackspec,*) 'la t ', trim(adjustl(title))
       write(lunbackspec,*) 'la f'
       write(lunbackspec,*) 'la x Energy (keV)'
       write(lunbackspec,*) 'la y Counts'
       write(lunbackspec,*) 'log x on'
       write(lunbackspec,*) 'log y on'
       write(lunbackspec,*) 'r x 0.2 15'
       do ichan=1,chansO
          if(maskFinal(ichan) .eq. 0.0) then ! OLD: mask
             specFinal(ichan) = 0.0          ! OLD: spec
             specFinalU(ichan) = 0.0         ! OLD: uncr
          endif
          write(lunbackspec,*) energy(ichan),specFinal(ichan),specFinalU(ichan),maskFinal(ichan)
       enddo
       close(lunbackspec)
    end if !  end SECTION 13

    ! SS: Read in the observation images (normal and OOT) in detector 
    !  coordinates - only if band processing
    ! OLDNEW: pnS001-obj-im-det-350-1100.fits/pnS001-fovimdet-350-1100.fits
    ! 780x780 Int32 image array
    ! SECTION 14
    !    print*,'Section 14'
    if (withimg) then
       !   SS: Read in images 
       !   BHP: below does all the lines of fitsio do
       status = 0 ! inIMG_F pnS001-fovimdet-elow-ehigh.fits
       warndie = .TRUE. ! croak if not found or unreadable
       call check_open_set(inIMG_F,inIMG_S,1,status,warndie) ! goflib/gofio_mod
       call read_2Darray(inIMG_S,inIMG_A,aDR32,aDR64,aDI16,inIMG_JP,xSize,ySize,typeA)
       write(errstr,*)" Sum of counts in FOV image     = ",sum(inIMG_JP)
       call message(AppMsg,Always) ! change to Noisy???
       status = 0 ! inIMGO_F pnS001-fovimootdet-elow-ehigh.fits
       warndie = .TRUE. ! croak if not found or unreadable
       call check_open_set(inIMGO_F,inIMGO_S,1,status,warndie) ! goflib/gofio_mod
       call read_2Darray(inIMGO_S,inIMGO_A,aDR32,aDR64,aDI16,inIMGO_JP,xSize,ySize,typeA)
       write(errstr,*)" Sum of counts in FOV OOT image = ",sum(inIMGO_JP)
       call message(AppMsg,Always)!  change to Noisy???
       tot0 = 0.
       tot1 = 0.
       tot2 = 0.
       tot3 = 0.
       imR  = 0.
       imoR = 0.
       do j=0,xSize-1
          do jj=0,ySize-1 ! check indices???
             ! is this necessary??? check conversions vs not???
             imR  = real(inIMG_JP(j,jj))    ! convert in image to real for calcs
             imoR = real(inIMGO_JP(j,jj))   ! convert in OOT image to real for calcs
             tot0 = tot0 + imR              ! OLD: image
             imR  = imR - ootsca*imoR       ! OLD: image_oot
             tot1 = tot1 + imR              ! OLD: image
             tot2 = tot2 + imageFinal(j,jj) ! OLD: outim
             tot3 = tot3 + ootsca*imoR      ! OLD: image_oot
          enddo
       enddo
       ! change these to Verbose for release???
       write(errstr,*)'Total observation counts in band     : ',tot0
       call message(AppMsg,Always)    
       write(errstr,*)'Total OOT counts in band             : ',tot3
       call message(AppMsg,Always)
       write(errstr,*)'Total OOT subtracted counts in band  : ',tot1
       call message(AppMsg,Always)
       write(errstr,*)'Total background counts in band      : ',tot2
       call message(AppMsg,Always)
       !
       ! SS: Write out the background image
       ! SS: Create the output background file name
       ! OLDNEW: pnS001-back-im-det-350-1100.fits/pnS001-bkgimdet-350-1100.fits
       write(errstr,*)'Creating background image in DET coords: ',trim(adjustl(outIMG_F))
       call message(AppMsg,Always)
       ! BHP: create a new fits file
       status = 0
       warndie = .TRUE.  ! croak if not found or unreadable
       call check_open_set(outIMG_F,outIMG_S,3,status,warndie) !  goflib/gofio_mod
       outIMG_A = addArray(outIMG_S,'PRIMARY',REAL32,dimensions=outDim)
       outIMG_RP => real32Array2Data(outIMG_A)
       do j=0,xSize-1
          do jj=0,ySize-1
             outIMG_RP(j,jj) = imageFinal(j+1,jj+1)
             !             if (imageFinal(j+1,jj+1) .ne. 0.) then 
             !                write(60,*)j,jj,imageFinal(j+1,jj+1),outIMG_RP(j,jj)
             !             end if
          end do
       end do
       ! write exposure???!!!
       call copyAllAttributes(attributable(inIMG_S),attributable(outIMG_S))
       call release(outIMG_S)
       call release(inIMGO_S)
       call release(inIMG_S)
    endif

    ! SECTION 15
    !    print*,'Section 15'
    if (withqdp) then ! checks for clobber at start
       !    OLDNEW: pnS001-bridge-fit.qdp/pnS003-bridgefit.qdp      
       call open_ascii_file(qdpbridge_F,lunbridge)
       write(lunbridge,*) 'viewport 0.2 0.2 0.9 0.85'
       write(lunbridge,*) 'read serr 2'
       write(lunbridge,*) 'font roman'
       write(lunbridge,*) 'cs 1.8'
       write(title,*) trim(adjustl(obsidO))," -- PN",trim(adjustl(expidO))
       write(lunbridge,*) 'la t ', title
       write(lunbridge,*) 'la f'
       write(lunbridge,*) 'la x Energy (keV)'
       write(lunbridge,*) 'la y Counts'
       write(lunbridge,*) 'log x on'
       write(lunbridge,*) 'log y on'
       write(lunbridge,*) 'r x 0.5 15'
    end if ! end SECTION 15

    ! SECTION 16
    !    print*,'Section 16'
    allocate(modl(chansO),tempe(chansO),temps(chansO),tempu(chansO))
    modl  = 0.
    tempe = 0.
    temps = 0.
    tempu = 0.
    ii = 0
    write(errstr,*)'nels = ',nels
    call message(AppMsg,Always) ! change to Noisy???!!!
    write(errstr,*)'mine ',mine
    call message(AppMsg,Always)
    write(errstr,*)'maxe ',maxe
    call message(AppMsg,Always)

    do k=1,nels
       nnn = 0
       !       print*,'**************** k = ',k,' n = ',nnn,' ii = ',ii
       do i=1,2
          do ichan=mine(i,k),maxe(i,k)
             if((energy(ichan) .gt. 0.1) .and.  &   ! OLD: ener
                  (specFinal(ichan) .gt. 0.0)) then ! OLD: spec
                nnn = nnn + 1
                tempe(nnn) = alog(energy(ichan))
                temps(nnn) = alog(specFinal(ichan))
                tempu(nnn) = 0.2
                !      write(29,*)ichan,energy(ichan),specFinal(ichan),tempe(nnn),temps(nnn),tempu(nnn)
             endif
          enddo
       enddo
       nord = 3
       ! BHP: poly_fit is in gofmatrix in task goflib
       ! poly_fit returns coef

       write(errstr,*)'****** pre poly' ! change to Noisy???!!!
       call message(AppMsg,Noisy)
       !write(30,*)i,modl(ichan),energy(ichan),coef(i+1),alog(energy(ichan))

       call poly_fit(tempe,temps,tempu,nnn,nord,coef) ! gofmatrix/goflib

       write(errstr,*)'****** post poly coefs',(coef(i), i=1,nord+1)
       call message(AppMsg,Always) ! change to Noisy???!!!
       do ichan=mine(1,k),maxe(2,k) 
          do i=0,nord
             modl(ichan) = modl(ichan) + &
                  coef(i+1)*(alog(energy(ichan))**i)
             !      write(31,*)ii,ichan,modl(ichan),coef(i+1),energy(ichan)
          enddo
       enddo

       ii = ii + 1
    enddo

    deallocate(tempe,temps,tempu) ! no longer needed

    if (withqdp) then
       do ichan=1,chansO
          tp = 0.0
          if (modl(ichan) .gt. 0.0) tp = exp(modl(ichan))
          write(lunbridge,*) energy(ichan), specFinal(ichan),0.05*specFinal(ichan), tp
       enddo
       close(lunbridge)
    end if
    !   print*,'end Section 16'

    ! end SECTION 16

    ! SS: Finish the bridge
    ! SECTION 17
    !   print*,'Section 17'
    allocate(fillin(chansO))
    fillin = 0.
    !   print*,'plc_alk = ',plc_alk
    do ichan = plc_alk(1),plc_alk(2)
       fillin(ichan) = exp(modl(ichan))
       specFinal(ichan)  = fillin(ichan)      ! OLD: spec
       specFinalU(ichan) = fillin(ichan)/20.0 ! OLD: uncr
    enddo
    !   print*,'plc_hard = ',plc_hard
    do ichan = plc_hard(1),plc_hard(2)
       fillin(ichan) = exp(modl(ichan))
       specFinal(ichan) = fillin(ichan)
       specFinalU(ichan) = fillin(ichan)/20.0
    enddo
    ! again, looks ok except chans 31-61, and specFinal has
    ! values abouve channel 3213 whereas they were 0 in old
    !    do i=1,chansO
    !       write(33,*)i,fillin(i),specFinal(i),specFinalU(i),modl(i)
    !    end do
    ymin = 1.e8
    ymax = 0
    deallocate (modl)
    do ichan = 7,chansO
       !       write(60,*)ichan,obj_cnts(ichan),stat(ichan)
       !       write(69,*)ichan,energy(ichan),specFinal(ichan),specFinalU(ichan)
       if((energy(ichan) .gt. 0.2) .and. &
            (energy(ichan) .lt. 16.0)) then
          if (obj_cnts(ichan)-stat(ichan) .lt. ymin) & !   OLD: obj_cnts,stat
               ymin = obj_cnts(ichan)-stat(ichan)
          if (obj_cnts(ichan)+stat(ichan) .gt. ymax) & !  OLD: obj_cnts,stat
               ymax = obj_cnts(ichan)+stat(ichan)
          if (specFinal(ichan)-specFinalU(ichan) .lt. ymin) ymin = & ! OLD: spec,uncr
               specFinal(ichan) - specFinalU(ichan) ! spec, uncr
          if (specFinal(ichan)+specFinal(ichan) .gt. ymax) ymax = & ! OLD: spec,uncr
               specFinal(ichan) + specFinalU(ichan) ! OLD: spec,uncr
       endif
       ! not ok? old ymin went quickly to 0, ymax went quickly to 7577
       !         new ymin went slowly to  6, ymax went slowly to    438
       !       write(70,*)ichan,ymin,ymax,specFinal(ichan),specFinalU(ichan)
    enddo ! end SECTION 17
    !
    !     SECTION 18
    !     SS: Get three unused luns
    !     BHP: Get three coffins ready, I mean...open three output sets with DAL
    !          Only need two here because template obj spec file is still open from
    !          above (don't forget to call release???)
    !     SS: lunc is for the OOT suptracted spectrum - not needed for CSTAT option
    !     SS: Create the output background spectrum FITS file using the 
    !         original spectrum file as a template
    !     OLDNEW:  pnS001-back.pi/pnS001-bkg.pi
    !     use input SPEC as template
    !    print*,'Section 18'
    !     change to Noisy???
    write(errstr,*) 'Creating background spectral file(s) using the Observation ' //&
         'spectral file as a template'
    call message(AppMsg,Always)
    write(errstr,*) &
         'Observation spectral file name (template) :  ',trim(adjustl(inSPECO_F)) 
    call message(AppMsg,Always)
    write(errstr,*) &
         'Background spectral file name             :  ',trim(adjustl(outSPECO_F)) 
    call message(AppMsg,Always)
    if(chisqfl) then
       write(errstr,*) &
            'Background OOT spectral file name         :  ',trim(adjustl(outSPECOO_F)) 
    endif
    !
    !    SS:  Write the output spectrum keywords
    !    BHP: copyDataSet completely clones the template spec to the output spec
    !        including all keywords, columns, extensions, so no need to write
    !        them out but they can be updated/modified later
    !   SECTION 19
    !    print*,'Section 19'
    !
    !  finish the job (make the bkg spectrum)
    !
    status = 0
    warndie = .TRUE. ! die if can't open and write to the file
    call check_open_set(outSPECO_F,outSPECO_S,3,status,warndie)!  3='create'
    if (chisqfl) then
       status = 0
       warndie = .TRUE. ! die if can't open and write to the file
       call check_open_set(outSPECOO_F,outSPECOO_S,3,status,warndie) ! 3='create'
    endif

    !     add appropriate columns
    if (chisqfl) then
       write(errstr,*)&
            "Setting up output background spectra with POISSONIAN stats (chisqfl=T)"
       call message(AppMsg,Always)
       outSPECO_T   = addTable(outSPECO_S,"SPECTRUM",chansO)
       channelO_C   = addColumn(outSPECO_T,"CHANNEL",INTEGER16,"channel",comment="channel number")
       channelO_IP => int16Data(channelO_C)
       countsOR32_C = addColumn(outSPECO_T,"RATE",REAL32,"counts/s",comment="Spectral counts/s")
       countsO_RP  => real32Data(countsOR32_C)
       staterrO_C   = addColumn(outSPECO_T,"STAT_ERR",REAL32,"counts/s",comment="Gaussian error")
       staterrO_RP => real32Data(staterrO_C)
       ! now the oot
       outSPECOO_T   = addTable(outSPECOO_S,"SPECTRUM",chansO)
       channelOO_C   = addColumn(outSPECOO_T,"CHANNEL",INTEGER16,"channel",comment="channel number")
       channelOO_IP => int16Data(channelOO_C)
       countsOOR32_C = addColumn(outSPECOO_T,"RATE",REAL32,"counts/s",comment="Spectral counts/s")
       countsOO_RP  => real32Data(countsOOR32_C)
       staterrOO_C   = addColumn(outSPECOO_T,"STAT_ERR",REAL32,"counts/s",comment="Gaussian error")
       staterrOO_RP => real32Data(staterrOO_C)
    else
       write(errstr,*)"Setting up output background spectra with GAUSSIAN stats"
       call message(AppMsg,Always)
       outSPECO_T   = addTable(outSPECO_S,"SPECTRUM",chansO)
       channelO_C   = addColumn(outSPECO_T,"CHANNEL",INTEGER16,comment="channel number")
       channelO_IP => int16Data(channelO_C)
       countsOI32_C = addColumn(outSPECO_T,"COUNTS",INTEGER32,comment="Spectral counts")
       countsO_JP  => int32Data(countsOI32_C)
    endif
    !
    !  SS: Prepare the spectrum data
    !
    ! OLDNEW: pnS001-spec.qdp/pnS003-augspec.qdp
    call open_ascii_file(qdpspec_F,lunspec)
    write(lunspec,*) 'viewport 0.2 0.2 0.9 0.85'
    write(lunspec,*) 'la f'
    write(title,*) trim(adjustl(obsidO))," -- PN",trim(adjustl(expidO))
    write(lunspec,*) 'la t ', title
    write(lunspec,*) 'la x Energy (keV)'
    write(lunspec,*) 'la y Counts'
    write(lunspec,*) 'la 1 p 2 ', 0.5*ymax, ' just left ', &
         '"Observed (red), Particle Background (green)"'
    if (.not.chisqfl) then
       write(lunspec,*) 'la 2 p 2 ', 0.3*ymax, ' just left ', &
            '"OOT Background (blue)"'
    endif
    write(lunspec,*) 'log y'
    write(lunspec,*) 'r x 0 15'
    write(lunspec,*) 'r y 0.1'
    write(lunspec,*) 'read serr 2'
    write(lunspec,*) 'read serr 3'
    write(lunspec,*) 'read serr'
    write(lunspec,*) 'font roman'
    write(lunspec,*) 'cs 1.8'
    !
    if(chisqfl) then
       do ichan=1,chansO
          channelO_IP(ichan-1) = ichan - 1
          countsO_RP(ichan-1)  = specFinal(ichan)/expoO
          staterrO_RP(ichan-1) = specFinalU(ichan)/expoO
          ! and the OOT
          channelOO_IP(ichan-1) = ichan - 1
          countsOO_RP(ichan-1)  = obj_cnts(ichan)/expoO
          staterrOO_RP(ichan-1) = stat(ichan)/expoO
          write(lunspec,*) energy(ichan), obj_cnts(ichan), &
               stat(ichan), specFinal(ichan), specFinalU(ichan)
       enddo
       call copyAllAttributes(attributable(inSPECO_S),attributable(outSPECO_S))
       call copyAllAttributes(attributable(inSPECO_T),attributable(outSPECO_T))
       call setAttribute(outSPECO_S,"POISSERR",.FALSE.)
       call setAttribute(outSPECO_S,"EXPOSURE",expoO,'Background exposure','s')
       ! and the OOT
       call copyAllAttributes(attributable(inSPECOO_S),attributable(outSPECOO_S))
       call copyAllAttributes(attributable(inSPECOO_T),attributable(outSPECOO_T))
       call setAttribute(outSPECOO_S,"POISSERR",.FALSE.)
       call setAttribute(outSPECOO_S,"EXPOSURE",expoO,'Background exposure','s')
       call release(outSPECOO_S)
    else
       do ichan=1,chansO
          channelO_IP(ichan-1) = ichan - 1
          countsO_JP(ichan-1) = nint(specFinal(ichan))
          write(lunspec,*) energy(ichan), obj_cnts(ichan), &
               stat(ichan), specFinal(ichan), &
               specFinal(ichan)/20.0, ootspec(ichan)
       enddo
       call copyAllAttributes(attributable(inSPECO_S),attributable(outSPECO_S))
       call copyAllAttributes(attributable(inSPECO_T),attributable(outSPECO_T))
       call setAttribute(outSPECO_S,"POISSERR",.TRUE.)
       call setAttribute(outSPECO_S,"EXPOSURE",expoN,'Background exposure','s')
    endif

    close(lunspec)
    call release(outSPECO_S)
    call release(inSPECO_S)
    call release(inSPECOO_T) 
    call release(inSPECOO_S) 

    return    

  end subroutine pnback
  !
  ! 
  !
  subroutine printparams (inSPEC_F,inOOT_F,outSPEC_F,outSPECO_F,inRMF_F,withqdp,&
       withimg,inIMG_F,inIMGO_F,outIMG_F,elow,ehigh,quads,rev,obsid,chisqfl)

    character(len=*) :: inSPEC_F, outSPECO_F, inOOT_F, inRMF_F, outSPEC_F, &
         inIMG_F, inIMGO_F, outIMG_F, obsid
    character(len=1) :: str_ccd
    integer :: elow, ehigh, rev, i
    logical :: withqdp, withimg, chisqfl, quads(4)

    write(errstr,*) "Input parameters:"
    call message(AppMsg,Always)
    write(errstr,*) "With chi-squared statistics? : ",chisqfl
    call message(AppMsg,Always)
    write(errstr,*) "Input Spectral file       : ",trim(inSPEC_F) ! old obj_file
    call message(AppMsg,Always)
    write(errstr,*) "Input OOT Spectral file   : ",trim(inOOT_F)  ! OOT file
    call message(AppMsg,Always)
    write(errstr,*) "Output Spectral file      : ",trim(outSPEC_F)! old back_file
    call message(AppMsg,Always)
    if (chisqfl) then
       write(errstr,*) "Output OOT Spectral file  : ",trim(outSPECO_F)! old back_file
       call message(AppMsg,Always)
    endif
    write(errstr,*) "Input RMF file            : ",trim(inRMF_F)  ! old obj_rmf
    call message(AppMsg,Always)
    write(errstr,*) "Create diagnostic IMAGE file (y/n)?: ",withimg
    call message(AppMsg,Always)
    if(withimg) then
       write(errstr,*) "Input IMAGE file        : ",trim(inIMG_F) ! old imo_name
       call message(AppMsg,Always)
       write(errstr,*) "Input OOT IMAGE file    : ",trim(inIMGO_F) ! old imo_name
       call message(AppMsg,Always)
       write(errstr,*) "Output IMAGE file       : ",trim(outIMG_F) ! old imo_name
       call message(AppMsg,Always)
    endif
    write(errstr,*) "Create diagnostic plot files (y/n)?: ",withqdp
    call message(AppMsg,Always)
    !
    write(errstr,*) "Selected energy lower limit         : ",elow
    call message(AppMsg,Always)
    write(errstr,*) "Selected energy upper limit         : ",ehigh
    call message(AppMsg,Always)
    write(errstr,*) "Revolution                          : ",rev
    call message(AppMsg,Always)
    write(errstr,*) "OBSID                               :   ",trim(adjustl(obsid))
    call message(AppMsg,Always)
    do i = 1, 4
       write(str_ccd,'(I1)')i
       write(errstr,*) "QUAD(",str_ccd,") selected? (y/n): ",quads(i)
       call message(AppMsg,Always)
    end do

    return

  end subroutine printparams


end module pnback_mod

